# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/e2t/e2/e2.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterable, AsyncIterator, Dict, Iterable, Union

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class EncodingType(betterproto.Enum):
    """EncodingType determines encoding type for the response messages"""

    PROTO = 0
    ASN1_PER = 1
    ASN1_XER = 2


class ResponseStatus(betterproto.Enum):
    """ResponseStatus"""

    FAILED = 0
    SUCCEEDED = 1
    REJECTED = 2


class Cause(betterproto.Enum):
    """Cause is a failure cause"""

    CAUSE_UNKNOWN = 0
    CAUSE_MISC_CONTROL_PROCESSING_OVERLOAD = 1
    CAUSE_MISC_HARDWARE_FAILURE = 2
    CAUSE_MISC_OM_INTERVENTION = 3
    CAUSE_MISC_UNSPECIFIED = 4
    CAUSE_PROTOCOL_TRANSFER_SYNTAX_ERROR = 5
    CAUSE_PROTOCOL_ABSTRACT_SYNTAX_ERROR_REJECT = 6
    CAUSE_PROTOCOL_ABSTRACT_SYNTAX_ERROR_IGNORE_AND_NOTIFY = 7
    CAUSE_PROTOCOL_MESSAGE_NOT_COMPATIBLE_WITH_RECEIVER_STATE = 8
    CAUSE_PROTOCOL_SEMANTIC_ERROR = 9
    CAUSE_PROTOCOL_ABSTRACT_SYNTAX_ERROR_FALSELY_CONSTRUCTED_MESSAGE = 10
    CAUSE_PROTOCOL_UNSPECIFIED = 11
    CAUSE_RIC_RAN_FUNCTION_ID_INVALID = 12
    CAUSE_RIC_ACTION_NOT_SUPPORTED = 13
    CAUSE_RIC_EXCESSIVE_ACTIONS = 14
    CAUSE_RIC_DUPLICATE_ACTION = 15
    CAUSE_RIC_DUPLICATE_EVENT = 16
    CAUSE_RIC_FUNCTION_RESOURCE_LIMIT = 17
    CAUSE_RIC_REQUEST_ID_UNKNOWN = 18
    CAUSE_RIC_INCONSISTENT_ACTION_SUBSEQUENT_ACTION_SEQUENCE = 19
    CAUSE_RIC_CONTROL_MESSAGE_INVALID = 20
    CAUSE_RIC_CALL_PROCESS_ID_INVALID = 21
    CAUSE_RIC_UNSPECIFIED = 22
    CAUSE_RICSERVICE_FUNCTION_NOT_REQUIRED = 23
    CAUSE_RICSERVICE_EXCESSIVE_FUNCTIONS = 24
    CAUSE_RICSERVICE_RIC_RESOURCE_LIMIT = 25
    CAUSE_TRANSPORT_UNSPECIFIED = 26
    CAUSE_TRANSPORT_TRANSPORT_RESOURCE_UNAVAILABLE = 27


class ControlAckRequest(betterproto.Enum):
    # Optional RIC Control Acknowledgement is required
    ACK = 0
    # Optional RIC Control Acknowledgement is not required
    NO_ACK = 1
    # Optional RIC Control Acknowledgement is only required to report failure
    NACK = 2


@dataclass(eq=False, repr=False)
class ServiceModel(betterproto.Message):
    """ServiceModelInfo E2 service model information"""

    name: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class RequestHeader(betterproto.Message):
    """
    RequestHeader a common request header for all requests including encoding
    type, client/xApp/session info, ordering info, etc
    """

    encoding_type: "EncodingType" = betterproto.enum_field(1)
    service_model: "ServiceModel" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ResponseHeader(betterproto.Message):
    """
    ResponseHeader a common response header for all responses including
    encoding type, client/xApp/session info, ordering info, etc
    """

    encoding_type: "EncodingType" = betterproto.enum_field(1)
    service_model: "ServiceModel" = betterproto.message_field(2)
    response_status: "ResponseStatus" = betterproto.enum_field(3)


@dataclass(eq=False, repr=False)
class StreamRequest(betterproto.Message):
    """StreamRequest"""

    header: "RequestHeader" = betterproto.message_field(1)
    app_id: str = betterproto.string_field(2)
    instance_id: str = betterproto.string_field(3)
    subscription_id: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class StreamResponse(betterproto.Message):
    """StreamResponse"""

    header: "ResponseHeader" = betterproto.message_field(1)
    indication_header: bytes = betterproto.bytes_field(2)
    indication_message: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class ControlRequest(betterproto.Message):
    """ControlRequest E2 control request"""

    header: "RequestHeader" = betterproto.message_field(1)
    e2_node_id: str = betterproto.string_field(2)
    control_header: bytes = betterproto.bytes_field(3)
    control_message: bytes = betterproto.bytes_field(4)
    control_ack_request: "ControlAckRequest" = betterproto.enum_field(5)


@dataclass(eq=False, repr=False)
class ControlResponse(betterproto.Message):
    """ControlResponse E2 control response"""

    header: "ResponseHeader" = betterproto.message_field(1)
    control_acknowledge: "ControlAcknowledge" = betterproto.message_field(
        2, group="response"
    )
    control_failure: "ControlFailure" = betterproto.message_field(3, group="response")


@dataclass(eq=False, repr=False)
class ControlAcknowledge(betterproto.Message):
    """ControlAcknowledge control acknowledgement"""

    control_outcome: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class ControlFailure(betterproto.Message):
    """ControlFailure control failure message"""

    cause: "Cause" = betterproto.enum_field(1)
    control_outcome: bytes = betterproto.bytes_field(2)
    message: str = betterproto.string_field(3)


class E2TServiceStub(betterproto.ServiceStub):
    async def stream(
        self,
        request_iterator: Union[
            AsyncIterable["StreamRequest"], Iterable["StreamRequest"]
        ],
    ) -> AsyncIterator["StreamResponse"]:

        async for response in self._stream_stream(
            "/onos.e2t.e2.E2TService/Stream",
            request_iterator,
            StreamRequest,
            StreamResponse,
        ):
            yield response

    async def control(
        self,
        *,
        header: "RequestHeader" = None,
        e2_node_id: str = "",
        control_header: bytes = b"",
        control_message: bytes = b"",
        control_ack_request: "ControlAckRequest" = None,
    ) -> "ControlResponse":

        request = ControlRequest()
        if header is not None:
            request.header = header
        request.e2_node_id = e2_node_id
        request.control_header = control_header
        request.control_message = control_message
        request.control_ack_request = control_ack_request

        return await self._unary_unary(
            "/onos.e2t.e2.E2TService/Control", request, ControlResponse
        )


class E2TServiceBase(ServiceBase):
    async def stream(
        self, request_iterator: AsyncIterator["StreamRequest"]
    ) -> AsyncIterator["StreamResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def control(
        self,
        header: "RequestHeader",
        e2_node_id: str,
        control_header: bytes,
        control_message: bytes,
        control_ack_request: "ControlAckRequest",
    ) -> "ControlResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_stream(self, stream: grpclib.server.Stream) -> None:
        request_kwargs = {"request_iterator": stream.__aiter__()}

        await self._call_rpc_handler_server_stream(
            self.stream,
            stream,
            request_kwargs,
        )

    async def __rpc_control(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "header": request.header,
            "e2_node_id": request.e2_node_id,
            "control_header": request.control_header,
            "control_message": request.control_message,
            "control_ack_request": request.control_ack_request,
        }

        response = await self.control(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.e2t.e2.E2TService/Stream": grpclib.const.Handler(
                self.__rpc_stream,
                grpclib.const.Cardinality.STREAM_STREAM,
                StreamRequest,
                StreamResponse,
            ),
            "/onos.e2t.e2.E2TService/Control": grpclib.const.Handler(
                self.__rpc_control,
                grpclib.const.Cardinality.UNARY_UNARY,
                ControlRequest,
                ControlResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/e2sub/endpoint/endpoint.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, Dict, List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class EventType(betterproto.Enum):
    """Type of change"""

    NONE = 0
    ADDED = 1
    REMOVED = 3


@dataclass(eq=False, repr=False)
class TerminationEndpoint(betterproto.Message):
    """
    Termination is a record identifying the IP address and TCP port coordinates
    where the E2 termination service is available.
    """

    id: str = betterproto.string_field(1)
    revision: int = betterproto.uint64_field(2)
    ip: str = betterproto.string_field(3)
    port: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class Event(betterproto.Message):
    """Event is an end-point event"""

    type: "EventType" = betterproto.enum_field(1)
    endpoint: "TerminationEndpoint" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class AddTerminationRequest(betterproto.Message):
    """
    AddTerminationRequest is a request for adding a new termination point
    """

    endpoint: "TerminationEndpoint" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class AddTerminationResponse(betterproto.Message):
    """
    AddTerminationResponse is a response to adding a new termination point
    """

    pass


@dataclass(eq=False, repr=False)
class GetTerminationRequest(betterproto.Message):
    """
    GetTerminationRequest is a request for getting existing termination point
    """

    id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class GetTerminationResponse(betterproto.Message):
    """
    GetTerminationResponse is a response with invormation about a requested
    termination point
    """

    endpoint: "TerminationEndpoint" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class RemoveTerminationRequest(betterproto.Message):
    """RemoveTerminationRequest is a request for removing termination point"""

    id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class RemoveTerminationResponse(betterproto.Message):
    """
    RemoveTerminationResponse is a response to removing a termination point
    """

    pass


@dataclass(eq=False, repr=False)
class ListTerminationsRequest(betterproto.Message):
    """
    ListTerminationsRequest is a request to list all available E2 terminations
    """

    pass


@dataclass(eq=False, repr=False)
class ListTerminationsResponse(betterproto.Message):
    """
    ListTerminationsResponse is a response to list all available E2
    terminations
    """

    endpoints: List["TerminationEndpoint"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class WatchTerminationsRequest(betterproto.Message):
    """
    WatchTerminationsRequest is a request to receive a stream of all E2
    termination changes.
    """

    noreplay: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class WatchTerminationsResponse(betterproto.Message):
    """
    WatchTerminationsResponse is a response indicating a change in the
    available E2 termination end-points.
    """

    event: "Event" = betterproto.message_field(1)


class E2RegistryServiceStub(betterproto.ServiceStub):
    async def add_termination(
        self, *, endpoint: "TerminationEndpoint" = None
    ) -> "AddTerminationResponse":

        request = AddTerminationRequest()
        if endpoint is not None:
            request.endpoint = endpoint

        return await self._unary_unary(
            "/onos.e2sub.endpoint.E2RegistryService/AddTermination",
            request,
            AddTerminationResponse,
        )

    async def get_termination(self, *, id: str = "") -> "GetTerminationResponse":

        request = GetTerminationRequest()
        request.id = id

        return await self._unary_unary(
            "/onos.e2sub.endpoint.E2RegistryService/GetTermination",
            request,
            GetTerminationResponse,
        )

    async def remove_termination(self, *, id: str = "") -> "RemoveTerminationResponse":

        request = RemoveTerminationRequest()
        request.id = id

        return await self._unary_unary(
            "/onos.e2sub.endpoint.E2RegistryService/RemoveTermination",
            request,
            RemoveTerminationResponse,
        )

    async def list_terminations(self) -> "ListTerminationsResponse":

        request = ListTerminationsRequest()

        return await self._unary_unary(
            "/onos.e2sub.endpoint.E2RegistryService/ListTerminations",
            request,
            ListTerminationsResponse,
        )

    async def watch_terminations(
        self, *, noreplay: bool = False
    ) -> AsyncIterator["WatchTerminationsResponse"]:

        request = WatchTerminationsRequest()
        request.noreplay = noreplay

        async for response in self._unary_stream(
            "/onos.e2sub.endpoint.E2RegistryService/WatchTerminations",
            request,
            WatchTerminationsResponse,
        ):
            yield response


class E2RegistryServiceBase(ServiceBase):
    async def add_termination(
        self, endpoint: "TerminationEndpoint"
    ) -> "AddTerminationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_termination(self, id: str) -> "GetTerminationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def remove_termination(self, id: str) -> "RemoveTerminationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_terminations(self) -> "ListTerminationsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch_terminations(
        self, noreplay: bool
    ) -> AsyncIterator["WatchTerminationsResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_add_termination(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "endpoint": request.endpoint,
        }

        response = await self.add_termination(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get_termination(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "id": request.id,
        }

        response = await self.get_termination(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_remove_termination(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "id": request.id,
        }

        response = await self.remove_termination(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list_terminations(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.list_terminations(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_watch_terminations(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "noreplay": request.noreplay,
        }

        await self._call_rpc_handler_server_stream(
            self.watch_terminations,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.e2sub.endpoint.E2RegistryService/AddTermination": grpclib.const.Handler(
                self.__rpc_add_termination,
                grpclib.const.Cardinality.UNARY_UNARY,
                AddTerminationRequest,
                AddTerminationResponse,
            ),
            "/onos.e2sub.endpoint.E2RegistryService/GetTermination": grpclib.const.Handler(
                self.__rpc_get_termination,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetTerminationRequest,
                GetTerminationResponse,
            ),
            "/onos.e2sub.endpoint.E2RegistryService/RemoveTermination": grpclib.const.Handler(
                self.__rpc_remove_termination,
                grpclib.const.Cardinality.UNARY_UNARY,
                RemoveTerminationRequest,
                RemoveTerminationResponse,
            ),
            "/onos.e2sub.endpoint.E2RegistryService/ListTerminations": grpclib.const.Handler(
                self.__rpc_list_terminations,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListTerminationsRequest,
                ListTerminationsResponse,
            ),
            "/onos.e2sub.endpoint.E2RegistryService/WatchTerminations": grpclib.const.Handler(
                self.__rpc_watch_terminations,
                grpclib.const.Cardinality.UNARY_STREAM,
                WatchTerminationsRequest,
                WatchTerminationsResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/v2/configuration.proto, onos/config/v2/transaction.proto, onos/config/v2/value.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase


class ValueType(betterproto.Enum):
    """ValueType is the type for a value"""

    EMPTY = 0
    STRING = 1
    INT = 2
    UINT = 3
    BOOL = 4
    DECIMAL = 5
    FLOAT = 6
    BYTES = 7
    LEAFLIST_STRING = 8
    LEAFLIST_INT = 9
    LEAFLIST_UINT = 10
    LEAFLIST_BOOL = 11
    LEAFLIST_DECIMAL = 12
    LEAFLIST_FLOAT = 13
    LEAFLIST_BYTES = 14


class ConfigurationState(betterproto.Enum):
    """
    ConfigurationState is the configuration state of a configuration phase
    """

    # CONFIGURATION_PENDING indicates the configuration is PENDING
    CONFIGURATION_PENDING = 0
    # COMPLETE indicates the configuration is COMPLETE
    CONFIGURATION_COMPLETE = 2


class ConfigurationEventType(betterproto.Enum):
    """
    ConfigurationEventType configuration event types for configuration store
    """

    # CONFIGURATION_EVENT_UNKNOWN indicates unknown configuration store event
    CONFIGURATION_EVENT_UNKNOWN = 0
    # CONFIGURATION_CREATED indicates the configuration entry in the store is
    # created
    CONFIGURATION_CREATED = 1
    # CONFIGURATION_UPDATED indicates the configuration entry in the store is
    # updated
    CONFIGURATION_UPDATED = 2
    # CONFIGURATION_DELETED indicates the configuration entry in the store is
    # deleted
    CONFIGURATION_DELETED = 3
    # CONFIGURATION_REPLAYED
    CONFIGURATION_REPLAYED = 4


class TransactionState(betterproto.Enum):
    """TransactionState is the transaction state of a transaction phase"""

    # TRANSACTION_PENDING indicates the transaction is pending
    TRANSACTION_PENDING = 0
    # TRANSACTION_COMPLETE indicates the transaction is complete
    TRANSACTION_COMPLETE = 2
    # TRANSACTION_FAILED indicates the transaction failed
    TRANSACTION_FAILED = 3
    # TRANSACTION_VALIDATING indicates the transaction is in the validating state
    TRANSACTION_VALIDATING = 4
    # TRANSACTION_VALIDATED indicates the transaction is validated successfully
    TRANSACTION_VALIDATED = 5
    # TRANSACTION_VALIDATION_FAILED indicates the transaction validation is
    # failed
    TRANSACTION_VALIDATION_FAILED = 6


class TransactionPhase(betterproto.Enum):
    """TransactionPhase is the phase of a Transaction"""

    # TRANSACTION_CHANGE indicates the transaction has been requested
    TRANSACTION_CHANGE = 0
    # TRANSACTION_ROLLBACK indicates a rollback has been requested for the
    # transaction
    TRANSACTION_ROLLBACK = 1


class TransactionEventType(betterproto.Enum):
    """TransactionEventType transaction event types for transaction store"""

    TRANSACTION_EVENT_UNKNOWN = 0
    TRANSACTION_CREATED = 1
    TRANSACTION_UPDATED = 2
    TRANSACTION_DELETED = 3
    TRANSACTION_REPLAYED = 4


@dataclass(eq=False, repr=False)
class TypedValue(betterproto.Message):
    """TypedValue is a value represented as a byte array"""

    # 'bytes' is the bytes array
    bytes_: bytes = betterproto.bytes_field(1)
    # 'type' is the value type
    type: "ValueType" = betterproto.enum_field(2)
    # 'type_opts' is a set of type options
    type_opts: List[int] = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class PathValue(betterproto.Message):
    """PathValue is an individual Path/Value combination"""

    # 'path' is the path to change
    path: str = betterproto.string_field(1)
    # 'value' is the change value
    value: "TypedValue" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Configuration(betterproto.Message):
    """Configuration represents complete desired target configuration"""

    # 'id' is a unique configuration identifier
    id: str = betterproto.string_field(1)
    # 'target_id' is the target to which the desired target configuration applies
    target_id: str = betterproto.string_field(2)
    # 'target_version' is the version to which desired target configuration
    # applies
    target_version: str = betterproto.string_field(3)
    # 'target_type' is an optional target type to which to apply this desired
    # target configuration
    target_type: str = betterproto.string_field(4)
    # 'values' is a list of path/values to set
    values: List["PathValue"] = betterproto.message_field(5)
    # 'ConfigurationStatus' is the current lifecycle status of the configuration
    status: "ConfigurationStatus" = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class ConfigurationStatus(betterproto.Message):
    """ConfigurationStatus is the status of a Configuration"""

    # 'state' is the state of the transaction within a Phase
    state: "ConfigurationState" = betterproto.enum_field(1)
    # mastershipState mastership info
    mastership_state: "MastershipState" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class MastershipState(betterproto.Message):
    """Mastership state"""

    term: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class ConfigurationEvent(betterproto.Message):
    """ConfigurationEvent configuration store event"""

    # ConfigurationEventType configuration event type
    type: "ConfigurationEventType" = betterproto.enum_field(1)
    configuration: "Configuration" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Transaction(betterproto.Message):
    """Transaction refers to a multi-target transactional change"""

    # 'id' is the unique identifier of the transaction This field should be set
    # prior to persisting the object.
    id: str = betterproto.string_field(1)
    # 'index' is a monotonically increasing, globally unique index of the change
    # The index is provided by the store, is static and unique for each unique
    # change identifier, and should not be modified by client code.
    index: int = betterproto.uint64_field(2)
    # 'revision' is the change revision number The revision number is provided by
    # the store and should not be modified by client code. Each unique state of
    # the change will be assigned a unique revision number which can be used for
    # optimistic concurrency control when updating or deleting the change state.
    revision: int = betterproto.uint64_field(3)
    # 'status' is the current lifecycle status of the transaction
    status: "TransactionStatus" = betterproto.message_field(4)
    # 'created' is the time at which the transaction was created
    created: datetime = betterproto.message_field(5)
    # 'updated' is the time at which the transaction was last updated
    updated: datetime = betterproto.message_field(6)
    # 'changes' is a set of changes to apply to targets The list of changes
    # should contain only a single change per target/version pair.
    changes: List["Change"] = betterproto.message_field(7)
    # 'deleted' is a flag indicating whether this transaction is being deleted by
    # a snapshot
    deleted: bool = betterproto.bool_field(8)
    # 'dependency' is a reference to the transaction on which this transaction is
    # dependent
    dependency: "TransactionRef" = betterproto.message_field(9)
    # 'dependents' is a list of references to transactions that depend on this
    # transaction
    dependents: List["TransactionRef"] = betterproto.message_field(10)
    # 'username' is the name of the user that made the transaction
    username: str = betterproto.string_field(11)


@dataclass(eq=False, repr=False)
class TransactionRef(betterproto.Message):
    """TransactionRef is a reference to a transaction"""

    none: "betterproto_lib_google_protobuf.Empty" = betterproto.message_field(
        1, group="id"
    )
    transaction_id: str = betterproto.string_field(2, group="id")


@dataclass(eq=False, repr=False)
class Change(betterproto.Message):
    """Change represents a configuration change to a single target"""

    # 'target_id' is the identifier of the target to which this change applies
    target_id: str = betterproto.string_field(1)
    # 'target_version' is an optional target version to which to apply this
    # change
    target_version: str = betterproto.string_field(2)
    # 'target_type' is an optional target type to which to apply this change
    target_type: str = betterproto.string_field(3)
    # 'values' is a set of change values to apply
    values: List["ChangeValue"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class ChangeValue(betterproto.Message):
    """
    ChangeValue is an individual Path/Value and removed flag combination in a
    Change
    """

    # 'path' is the path to change
    path: str = betterproto.string_field(1)
    # 'value' is the change value
    value: "TypedValue" = betterproto.message_field(2)
    # 'removed' indicates whether this is a delete
    removed: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class TransactionStatus(betterproto.Message):
    """TransactionStatus is the status of a Transaction"""

    # 'phase' is the current phase of the
    phase: "TransactionPhase" = betterproto.enum_field(1)
    # 'state' is the state of the transaction within a Phase
    state: "TransactionState" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class TransactionEvent(betterproto.Message):
    """TransactionEvent transaction store event"""

    type: "TransactionEventType" = betterproto.enum_field(1)
    transaction: "Transaction" = betterproto.message_field(2)


import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/snapshot/network/types.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase


@dataclass(eq=False, repr=False)
class NetworkSnapshot(betterproto.Message):
    """NetworkSnapshot is a snapshot of all network changes"""

    # 'id' is the unique snapshot identifier
    id: str = betterproto.string_field(1)
    # 'index' is a monotonically increasing, globally unique snapshot index
    index: int = betterproto.uint64_field(2)
    # 'revision' is the request revision number
    revision: int = betterproto.uint64_field(3)
    # 'status' is the snapshot status
    status: "__snapshot__.Status" = betterproto.message_field(4)
    # 'retention' specifies the duration for which to retain changes
    retention: "__snapshot__.RetentionOptions" = betterproto.message_field(6)
    # 'created' is the time at which the configuration was created
    created: datetime = betterproto.message_field(8)
    # 'updated' is the time at which the configuration was last updated
    updated: datetime = betterproto.message_field(9)
    # 'refs' is a set of references to stored device snapshots
    refs: List["DeviceSnapshotRef"] = betterproto.message_field(10)


@dataclass(eq=False, repr=False)
class DeviceSnapshotRef(betterproto.Message):
    """DeviceSnapshotRef is a reference to a device snapshot"""

    # 'device_snapshot_id' is the unique identifier of the device snapshot
    device_snapshot_id: str = betterproto.string_field(1)


from ... import snapshot as __snapshot__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: github.com/openconfig/gnmi/proto/gnmi_ext/gnmi_ext.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase


class ExtensionId(betterproto.Enum):
    """
    RegisteredExtension is an enumeration acting as a registry for extensions
    defined by external sources.
    """

    EID_UNSET = 0
    # An experimental extension that may be used during prototyping of a new
    # extension.
    EID_EXPERIMENTAL = 999


@dataclass(eq=False, repr=False)
class Extension(betterproto.Message):
    """The Extension message contains a single gNMI extension."""

    registered_ext: "RegisteredExtension" = betterproto.message_field(1, group="ext")
    # Well known extensions.
    master_arbitration: "MasterArbitration" = betterproto.message_field(2, group="ext")
    history: "History" = betterproto.message_field(3, group="ext")


@dataclass(eq=False, repr=False)
class RegisteredExtension(betterproto.Message):
    """
    The RegisteredExtension message defines an extension which is defined
    outside of this file.
    """

    id: "ExtensionId" = betterproto.enum_field(1)
    msg: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class MasterArbitration(betterproto.Message):
    """
    MasterArbitration is used to select the master among multiple gNMI clients
    with the same Roles. The client with the largest election_id is honored as
    the master. The document about gNMI master arbitration can be found at
    https://github.com/openconfig/reference/blob/master/rpc/gnmi/gnmi-master-
    arbitration.md
    """

    role: "Role" = betterproto.message_field(1)
    election_id: "Uint128" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Uint128(betterproto.Message):
    """Representation of unsigned 128-bit integer."""

    high: int = betterproto.uint64_field(1)
    low: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class Role(betterproto.Message):
    """
    There can be one master for each role. The role is identified by its id.
    """

    id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class History(betterproto.Message):
    """
    The History extension allows clients to request historical data. Its spec
    can be found at
    https://github.com/openconfig/reference/blob/master/rpc/gnmi/gnmi-
    history.md
    """

    snapshot_time: int = betterproto.int64_field(1, group="request")
    range: "TimeRange" = betterproto.message_field(2, group="request")


@dataclass(eq=False, repr=False)
class TimeRange(betterproto.Message):
    start: int = betterproto.int64_field(1)
    end: int = betterproto.int64_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/admin/admin.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import timedelta
from typing import AsyncIterable, AsyncIterator, Dict, Iterable, List, Union

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib
import gnmi.proto

class Type(betterproto.Enum):
    """Streaming event type"""

    # NONE indicates this response does not represent a state change
    NONE = 0
    # ADDED is an event which occurs when an item is added
    ADDED = 1
    # UPDATED is an event which occurs when an item is updated
    UPDATED = 2
    # REMOVED is an event which occurs when an item is removed
    REMOVED = 3


@dataclass(eq=False, repr=False)
class ReadOnlySubPath(betterproto.Message):
    """
    ReadOnlySubPath is an extension to the ReadOnlyPath to define the datatype
    of the subpath
    """

    # sub_path is the relative path of a child object e.g. /list2b/index
    sub_path: str = betterproto.string_field(1)
    # value_type is the datatype of the read only path
    value_type: "_change_device__.ValueType" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class ReadOnlyPath(betterproto.Message):
    """
    ReadOnlyPath extracted from the model plugin as the definition of a tree of
    read only items. In YANG models items are defined as ReadOnly with the
    `config false` keyword. This can be applied to single items (leafs) or
    collections (containers or lists). When this `config false` is applied to
    an object every item beneath it will also become readonly - here these are
    shown as subpaths. The complete read only path then will be a concatenation
    of both e.g. /cont1a/cont1b-state/list2b/index and the type is defined in
    the SubPath as UInt8.
    """

    # path of the topmost `config false` object e.g. /cont1a/cont1b-state
    path: str = betterproto.string_field(1)
    # ReadOnlySubPath is a set of children of the path including an entry for the
    # type of the topmost object with subpath `/` An example is /list2b/index
    sub_path: List["ReadOnlySubPath"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ReadWritePath(betterproto.Message):
    """
    ReadWritePath is extracted from the model plugin as the definition of a
    writeable attributes. In YANG models items are writable by default unless
    they are specified as `config false` or have an item with `config false` as
    a parent. Each configurable item has metadata with meanings taken from the
    YANG specification RFC 6020.
    """

    # path is the full path to the attribute (leaf or leaf-list)
    path: str = betterproto.string_field(1)
    # value_type is the data type of the attribute
    value_type: "_change_device__.ValueType" = betterproto.enum_field(2)
    # units is the unit of measurement e.g. dB, mV
    units: str = betterproto.string_field(3)
    # description is an explaination of the meaning of the attribute
    description: str = betterproto.string_field(4)
    # mandatory shows whether the attribute is optional (false) or required
    # (true)
    mandatory: bool = betterproto.bool_field(5)
    # default is a default value used with optional attributes
    default: str = betterproto.string_field(6)
    # range is definition of the range of values a value is allowed
    range: List[str] = betterproto.string_field(7)
    # length is a defintion of the length restrictions for the attribute
    length: List[str] = betterproto.string_field(8)


@dataclass(eq=False, repr=False)
class ModelInfo(betterproto.Message):
    """ModelInfo is general information about a model plugin."""

    # name is the name given to the model plugin - no spaces and title case.
    name: str = betterproto.string_field(1)
    # version is the semantic version of the Plugin e.g. 1.0.0.
    version: str = betterproto.string_field(2)
    # model_data is a set of metadata about the YANG files that went in to
    # generating the model plugin. It includes name, version and organization for
    # each YANG file, similar to how they are represented in gNMI Capabilities.
    model_data: List["gnmi.proto.ModelData"] = betterproto.message_field(3)
    # module is the name of the Model Plugin on the file system - usually ending
    # in .so.<version>.
    module: str = betterproto.string_field(4)
    # getStateMode is flag that defines how the "get state" operation works.  0)
    # means that no retrieval of state is attempted  1) means that the
    # synchronizer will make 2 requests to the device - one for      Get with
    # State and another for Get with Operational.  2) means that the synchronizer
    # will do a Get request comprising of each      one of the ReadOnlyPaths and
    # their sub paths. If there is a `list`      in any one of these paths it
    # will be sent down as is, expecting the      devices implementation of gNMI
    # will be able to expand wildcards.  3) means that the synchronizer will do a
    # Get request comprising of each      one of the ReadOnlyPaths and their sub
    # paths. If there is a `list`      in any one of these paths, a separate call
    # will be made first to find      all the instances in the list and a Get
    # including these expanded wildcards      will be sent down to the device.
    get_state_mode: int = betterproto.uint32_field(5)
    # read_only_path is all of the read only paths for the model plugin.
    read_only_path: List["ReadOnlyPath"] = betterproto.message_field(7)
    # read_write_path is all of the read write paths for the model plugin.
    read_write_path: List["ReadWritePath"] = betterproto.message_field(8)


@dataclass(eq=False, repr=False)
class Chunk(betterproto.Message):
    """
    Chunk is for streaming a model plugin file to the server. There is a built
    in limit in gRPC of 4MB - plugin is usually around 20MB so break in to
    chunks of approx 1-2MB.
    """

    # so_file is the name being streamed.
    so_file: str = betterproto.string_field(1)
    # content is the bytes content.
    content: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class RegisterResponse(betterproto.Message):
    """RegisterResponse carries status of model plugin registration."""

    # name is name of the model plugin.
    name: str = betterproto.string_field(1)
    # version is the semantic version of the model plugin.
    version: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ListModelsRequest(betterproto.Message):
    """
    ListModelsRequest carries data for querying registered model plugins.
    """

    # verbose option causes all of the ReadWrite and ReadOnly paths to be
    # included.
    verbose: bool = betterproto.bool_field(1)
    # An optional filter on the name of the model plugins to list.
    model_name: str = betterproto.string_field(2)
    # An optional filter on the version of the model plugins to list
    model_version: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class RollbackRequest(betterproto.Message):
    """
    RollbackRequest carries the name of a network config to rollback. If there
    are subsequent changes to any of the devices in that config, the rollback
    will be rejected.
    """

    # name is an optional name of a Network Change to rollback. If no name is
    # given the last network change will be rolled back. If the name given is not
    # of the last network change and error will be given.
    name: str = betterproto.string_field(1)
    # On optional comment to leave on the rollback.
    comment: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class RollbackResponse(betterproto.Message):
    """RollbackResponse carries the response of the rollback operation"""

    # A message showing the result of the rollback.
    message: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ListSnapshotsRequest(betterproto.Message):
    """
    ListSnapshotsRequest requests a list of snapshots for all devices and
    versions.
    """

    # subscribe indicates whether to subscribe to events (e.g. ADD, UPDATE, and
    # REMOVE) that occur after all devices have been streamed to the client
    subscribe: bool = betterproto.bool_field(1)
    # option to specify a specific device - if blank or '*' then select all Can
    # support `*` (match many chars) or '?' (match one char) as wildcard
    id: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CompactChangesRequest(betterproto.Message):
    """
    CompactChangesRequest requests a compaction of the Network Change and
    Device Change stores
    """

    # retention_period is an optional duration of time counting back from the
    # present moment Network changes that were created during this period should
    # not be compacted Any network changes that are older should be compacted If
    # not specified the duration is 0
    retention_period: timedelta = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CompactChangesResponse(betterproto.Message):
    """CompactChangesResponse is a response to the Compact Changes command"""

    pass


@dataclass(eq=False, repr=False)
class ModelInfoRequest(betterproto.Message):
    """ModelInfoRequest carries request for the model information"""

    pass


@dataclass(eq=False, repr=False)
class ModelInfoResponse(betterproto.Message):
    """ModelInfoResponse carries response for the model information query"""

    model_info: "ModelInfo" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ValidateConfigRequest(betterproto.Message):
    """
    ValidateConfigRequest carries configuration data to be validated as a JSON
    blob
    """

    json: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class ValidateConfigResponse(betterproto.Message):
    """ValidateConfigResponse carries the result of the validation"""

    valid: bool = betterproto.bool_field(1)
    message: str = betterproto.string_field(2)


class ConfigAdminServiceStub(betterproto.ServiceStub):
    async def upload_register_model(
        self, request_iterator: Union[AsyncIterable["Chunk"], Iterable["Chunk"]]
    ) -> "RegisterResponse":

        return await self._stream_unary(
            "/onos.config.admin.ConfigAdminService/UploadRegisterModel",
            request_iterator,
            Chunk,
            RegisterResponse,
        )

    async def list_registered_models(
        self, *, verbose: bool = False, model_name: str = "", model_version: str = ""
    ) -> AsyncIterator["ModelInfo"]:

        request = ListModelsRequest()
        request.verbose = verbose
        request.model_name = model_name
        request.model_version = model_version

        async for response in self._unary_stream(
            "/onos.config.admin.ConfigAdminService/ListRegisteredModels",
            request,
            ModelInfo,
        ):
            yield response

    async def rollback_network_change(
        self, *, name: str = "", comment: str = ""
    ) -> "RollbackResponse":

        request = RollbackRequest()
        request.name = name
        request.comment = comment

        return await self._unary_unary(
            "/onos.config.admin.ConfigAdminService/RollbackNetworkChange",
            request,
            RollbackResponse,
        )

    async def list_snapshots(
        self, *, subscribe: bool = False, id: str = ""
    ) -> AsyncIterator["_snapshot_device__.Snapshot"]:

        request = ListSnapshotsRequest()
        request.subscribe = subscribe
        request.id = id

        async for response in self._unary_stream(
            "/onos.config.admin.ConfigAdminService/ListSnapshots",
            request,
            _snapshot_device__.Snapshot,
        ):
            yield response

    async def compact_changes(
        self, *, retention_period: timedelta = None
    ) -> "CompactChangesResponse":

        request = CompactChangesRequest()
        if retention_period is not None:
            request.retention_period = retention_period

        return await self._unary_unary(
            "/onos.config.admin.ConfigAdminService/CompactChanges",
            request,
            CompactChangesResponse,
        )


class ModelPluginServiceStub(betterproto.ServiceStub):
    async def get_model_info(self) -> "ModelInfoResponse":

        request = ModelInfoRequest()

        return await self._unary_unary(
            "/onos.config.admin.ModelPluginService/GetModelInfo",
            request,
            ModelInfoResponse,
        )

    async def validate_config(self, *, json: bytes = b"") -> "ValidateConfigResponse":

        request = ValidateConfigRequest()
        request.json = json

        return await self._unary_unary(
            "/onos.config.admin.ModelPluginService/ValidateConfig",
            request,
            ValidateConfigResponse,
        )


class ConfigAdminServiceBase(ServiceBase):
    async def upload_register_model(
        self, request_iterator: AsyncIterator["Chunk"]
    ) -> "RegisterResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_registered_models(
        self, verbose: bool, model_name: str, model_version: str
    ) -> AsyncIterator["ModelInfo"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def rollback_network_change(
        self, name: str, comment: str
    ) -> "RollbackResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_snapshots(
        self, subscribe: bool, id: str
    ) -> AsyncIterator["_snapshot_device__.Snapshot"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def compact_changes(
        self, retention_period: timedelta
    ) -> "CompactChangesResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_upload_register_model(self, stream: grpclib.server.Stream) -> None:
        request_kwargs = {"request_iterator": stream.__aiter__()}

        response = await self.upload_register_model(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list_registered_models(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "verbose": request.verbose,
            "model_name": request.model_name,
            "model_version": request.model_version,
        }

        await self._call_rpc_handler_server_stream(
            self.list_registered_models,
            stream,
            request_kwargs,
        )

    async def __rpc_rollback_network_change(
        self, stream: grpclib.server.Stream
    ) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "name": request.name,
            "comment": request.comment,
        }

        response = await self.rollback_network_change(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list_snapshots(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "subscribe": request.subscribe,
            "id": request.id,
        }

        await self._call_rpc_handler_server_stream(
            self.list_snapshots,
            stream,
            request_kwargs,
        )

    async def __rpc_compact_changes(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "retention_period": request.retention_period,
        }

        response = await self.compact_changes(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.config.admin.ConfigAdminService/UploadRegisterModel": grpclib.const.Handler(
                self.__rpc_upload_register_model,
                grpclib.const.Cardinality.STREAM_UNARY,
                Chunk,
                RegisterResponse,
            ),
            "/onos.config.admin.ConfigAdminService/ListRegisteredModels": grpclib.const.Handler(
                self.__rpc_list_registered_models,
                grpclib.const.Cardinality.UNARY_STREAM,
                ListModelsRequest,
                ModelInfo,
            ),
            "/onos.config.admin.ConfigAdminService/RollbackNetworkChange": grpclib.const.Handler(
                self.__rpc_rollback_network_change,
                grpclib.const.Cardinality.UNARY_UNARY,
                RollbackRequest,
                RollbackResponse,
            ),
            "/onos.config.admin.ConfigAdminService/ListSnapshots": grpclib.const.Handler(
                self.__rpc_list_snapshots,
                grpclib.const.Cardinality.UNARY_STREAM,
                ListSnapshotsRequest,
                _snapshot_device__.Snapshot,
            ),
            "/onos.config.admin.ConfigAdminService/CompactChanges": grpclib.const.Handler(
                self.__rpc_compact_changes,
                grpclib.const.Cardinality.UNARY_UNARY,
                CompactChangesRequest,
                CompactChangesResponse,
            ),
        }


class ModelPluginServiceBase(ServiceBase):
    async def get_model_info(self) -> "ModelInfoResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def validate_config(self, json: bytes) -> "ValidateConfigResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_model_info(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_model_info(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_validate_config(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "json": request.json,
        }

        response = await self.validate_config(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.config.admin.ModelPluginService/GetModelInfo": grpclib.const.Handler(
                self.__rpc_get_model_info,
                grpclib.const.Cardinality.UNARY_UNARY,
                ModelInfoRequest,
                ModelInfoResponse,
            ),
            "/onos.config.admin.ModelPluginService/ValidateConfig": grpclib.const.Handler(
                self.__rpc_validate_config,
                grpclib.const.Cardinality.UNARY_UNARY,
                ValidateConfigRequest,
                ValidateConfigResponse,
            ),
        }


from ..change import device as _change_device__
from ..snapshot import device as _snapshot_device__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: github.com/openconfig/gnmi/proto/gnmi/gnmi.proto
# plugin: python-betterproto
import warnings
from dataclasses import dataclass
from typing import AsyncIterable, AsyncIterator, Dict, Iterable, List, Optional, Union

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class Encoding(betterproto.Enum):
    """
    Encoding defines the value encoding formats that are supported by the gNMI
    protocol. These encodings are used by both the client (when sending Set
    messages to modify the state of the target) and the target when serializing
    data to be returned to the client (in both Subscribe and Get RPCs).
    Reference: gNMI Specification Section 2.3
    """

    JSON = 0
    BYTES = 1
    PROTO = 2
    ASCII = 3
    JSON_IETF = 4


class SubscriptionMode(betterproto.Enum):
    """
    SubscriptionMode is the mode of the subscription, specifying how the target
    must return values in a subscription. Reference: gNMI Specification Section
    3.5.1.3
    """

    TARGET_DEFINED = 0
    ON_CHANGE = 1
    SAMPLE = 2


class SubscriptionListMode(betterproto.Enum):
    STREAM = 0
    ONCE = 1
    POLL = 2


class UpdateResultOperation(betterproto.Enum):
    INVALID = 0
    DELETE = 1
    REPLACE = 2
    UPDATE = 3


class GetRequestDataType(betterproto.Enum):
    ALL = 0
    CONFIG = 1
    STATE = 2
    OPERATIONAL = 3


@dataclass(eq=False, repr=False)
class Notification(betterproto.Message):
    """
    Notification is a re-usable message that is used to encode data from the
    target to the client. A Notification carries two types of changes to the
    data tree:  - Deleted values (delete) - a set of paths that have been
    removed from the    data tree.  - Updated values (update) - a set of path-
    value pairs indicating the path    whose value has changed in the data
    tree. Reference: gNMI Specification Section 2.1
    """

    timestamp: int = betterproto.int64_field(1)
    prefix: "Path" = betterproto.message_field(2)
    # An alias for the path specified in the prefix field. Reference: gNMI
    # Specification Section 2.4.2
    alias: str = betterproto.string_field(3)
    update: List["Update"] = betterproto.message_field(4)
    delete: List["Path"] = betterproto.message_field(5)
    # This notification contains a set of paths that are always updated together
    # referenced by a globally unique prefix.
    atomic: bool = betterproto.bool_field(6)


@dataclass(eq=False, repr=False)
class Update(betterproto.Message):
    """
    Update is a re-usable message that is used to store a particular Path,
    Value pair. Reference: gNMI Specification Section 2.1
    """

    path: "Path" = betterproto.message_field(1)
    value: "Value" = betterproto.message_field(2)
    val: "TypedValue" = betterproto.message_field(3)
    duplicates: int = betterproto.uint32_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.value:
            warnings.warn("Update.value is deprecated", DeprecationWarning)


@dataclass(eq=False, repr=False)
class TypedValue(betterproto.Message):
    """
    TypedValue is used to encode a value being sent between the client and
    target (originated by either entity).
    """

    string_val: str = betterproto.string_field(1, group="value")
    int_val: int = betterproto.int64_field(2, group="value")
    uint_val: int = betterproto.uint64_field(3, group="value")
    bool_val: bool = betterproto.bool_field(4, group="value")
    bytes_val: bytes = betterproto.bytes_field(5, group="value")
    float_val: float = betterproto.float_field(6, group="value")
    decimal_val: "Decimal64" = betterproto.message_field(7, group="value")
    leaflist_val: "ScalarArray" = betterproto.message_field(8, group="value")
    any_val: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(
        9, group="value"
    )
    json_val: bytes = betterproto.bytes_field(10, group="value")
    json_ietf_val: bytes = betterproto.bytes_field(11, group="value")
    ascii_val: str = betterproto.string_field(12, group="value")
    # Protobuf binary encoded bytes. The message type is not included. See the
    # specification at
    # github.com/openconfig/reference/blob/master/rpc/gnmi/protobuf-vals.md for a
    # complete specification.
    proto_bytes: bytes = betterproto.bytes_field(13, group="value")


@dataclass(eq=False, repr=False)
class Path(betterproto.Message):
    """
    Path encodes a data tree path as a series of repeated strings, with each
    element of the path representing a data tree node name and the associated
    attributes. Reference: gNMI Specification Section 2.2.2.
    """

    # Elements of the path are no longer encoded as a string, but rather within
    # the elem field as a PathElem message.
    element: List[str] = betterproto.string_field(1)
    origin: str = betterproto.string_field(2)
    elem: List["PathElem"] = betterproto.message_field(3)
    target: str = betterproto.string_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.element:
            warnings.warn("Path.element is deprecated", DeprecationWarning)


@dataclass(eq=False, repr=False)
class PathElem(betterproto.Message):
    """
    PathElem encodes an element of a gNMI path, along with any attributes
    (keys) that may be associated with it. Reference: gNMI Specification
    Section 2.2.2.
    """

    name: str = betterproto.string_field(1)
    key: Dict[str, str] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )


@dataclass(eq=False, repr=False)
class Value(betterproto.Message):
    """
    Value encodes a data tree node's value - along with the way in which the
    value is encoded. This message is deprecated by gNMI 0.3.0. Reference: gNMI
    Specification Section 2.2.3.
    """

    value: bytes = betterproto.bytes_field(1)
    type: "Encoding" = betterproto.enum_field(2)

    def __post_init__(self) -> None:
        warnings.warn("Value is deprecated", DeprecationWarning)
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Error(betterproto.Message):
    """
    Error message previously utilised to return errors to the client.
    Deprecated in favour of using the
    google.golang.org/genproto/googleapis/rpc/status message in the RPC
    response. Reference: gNMI Specification Section 2.5
    """

    code: int = betterproto.uint32_field(1)
    message: str = betterproto.string_field(2)
    data: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(3)

    def __post_init__(self) -> None:
        warnings.warn("Error is deprecated", DeprecationWarning)
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Decimal64(betterproto.Message):
    """
    Decimal64 is used to encode a fixed precision decimal number. The value is
    expressed as a set of digits with the precision specifying the number of
    digits following the decimal point in the digit set.
    """

    digits: int = betterproto.int64_field(1)
    precision: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class ScalarArray(betterproto.Message):
    """ScalarArray is used to encode a mixed-type array of values."""

    # The set of elements within the array. Each TypedValue message should
    # specify only elements that have a field identifier of 1-7 (i.e., the values
    # are scalar values).
    element: List["TypedValue"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class SubscribeRequest(betterproto.Message):
    """
    SubscribeRequest is the message sent by the client to the target when
    initiating a subscription to a set of paths within the data tree. The
    request field must be populated and the initial message must specify a
    SubscriptionList to initiate a subscription. The message is subsequently
    used to define aliases or trigger polled data to be sent by the target.
    Reference: gNMI Specification Section 3.5.1.1
    """

    subscribe: "SubscriptionList" = betterproto.message_field(1, group="request")
    poll: "Poll" = betterproto.message_field(3, group="request")
    aliases: "AliasList" = betterproto.message_field(4, group="request")
    # Extension messages associated with the SubscribeRequest. See the gNMI
    # extension specification for further definition.
    extension: List["_gnmi_ext__.Extension"] = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class Poll(betterproto.Message):
    """
    Poll is sent within a SubscribeRequest to trigger the device to send
    telemetry updates for the paths that are associated with the subscription.
    Reference: gNMI Specification Section Section 3.5.1.4
    """

    pass


@dataclass(eq=False, repr=False)
class SubscribeResponse(betterproto.Message):
    """
    SubscribeResponse is the message used by the target within a Subscribe RPC.
    The target includes a Notification message which is used to transmit values
    of the path(s) that are associated with the subscription. The same message
    is to indicate that the target has sent all data values once (is
    synchronized). Reference: gNMI Specification Section 3.5.1.4
    """

    update: "Notification" = betterproto.message_field(1, group="response")
    # Indicate target has sent all values associated with the subscription at
    # least once.
    sync_response: bool = betterproto.bool_field(3, group="response")
    # Deprecated in favour of google.golang.org/genproto/googleapis/rpc/status
    error: "Error" = betterproto.message_field(4, group="response")
    # Extension messages associated with the SubscribeResponse. See the gNMI
    # extension specification for further definition.
    extension: List["_gnmi_ext__.Extension"] = betterproto.message_field(5)

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.error:
            warnings.warn("SubscribeResponse.error is deprecated", DeprecationWarning)


@dataclass(eq=False, repr=False)
class SubscriptionList(betterproto.Message):
    """
    SubscriptionList is used within a Subscribe message to specify the list of
    paths that the client wishes to subscribe to. The message consists of a
    list of (possibly prefixed) paths, and options that relate to the
    subscription. Reference: gNMI Specification Section 3.5.1.2
    """

    prefix: "Path" = betterproto.message_field(1)
    subscription: List["Subscription"] = betterproto.message_field(2)
    # Whether target defined aliases are allowed within the subscription.
    use_aliases: bool = betterproto.bool_field(3)
    qos: "QosMarking" = betterproto.message_field(4)
    mode: "SubscriptionListMode" = betterproto.enum_field(5)
    # Whether elements of the schema that are marked as eligible for aggregation
    # should be aggregated or not.
    allow_aggregation: bool = betterproto.bool_field(6)
    # The set of schemas that define the elements of the data tree that should be
    # sent by the target.
    use_models: List["ModelData"] = betterproto.message_field(7)
    # The encoding that the target should use within the Notifications generated
    # corresponding to the SubscriptionList.
    encoding: "Encoding" = betterproto.enum_field(8)
    # An optional field to specify that only updates to current state should be
    # sent to a client. If set, the initial state is not sent to the client but
    # rather only the sync message followed by any subsequent updates to the
    # current state. For ONCE and POLL modes, this causes the server to send only
    # the sync message (Sec. 3.5.2.3).
    updates_only: bool = betterproto.bool_field(9)


@dataclass(eq=False, repr=False)
class Subscription(betterproto.Message):
    """
    Subscription is a single request within a SubscriptionList. The path
    specified is interpreted (along with the prefix) as the elements of the
    data tree that the client is subscribing to. The mode determines how the
    target should trigger updates to be sent. Reference: gNMI Specification
    Section 3.5.1.3
    """

    path: "Path" = betterproto.message_field(1)
    mode: "SubscriptionMode" = betterproto.enum_field(2)
    sample_interval: int = betterproto.uint64_field(3)
    # Indicates whether values that have not changed should be sent in a SAMPLE
    # subscription.
    suppress_redundant: bool = betterproto.bool_field(4)
    # Specifies the maximum allowable silent period in nanoseconds when
    # suppress_redundant is in use. The target should send a value at least once
    # in the period specified.
    heartbeat_interval: int = betterproto.uint64_field(5)


@dataclass(eq=False, repr=False)
class QosMarking(betterproto.Message):
    """
    QOSMarking specifies the DSCP value to be set on transmitted telemetry
    updates from the target. Reference: gNMI Specification Section 3.5.1.2
    """

    marking: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class Alias(betterproto.Message):
    """
    Alias specifies a data tree path, and an associated string which defines an
    alias which is to be used for this path in the context of the RPC. The
    alias is specified as a string which is prefixed with "#" to disambiguate
    it from data tree element paths. Reference: gNMI Specification Section
    2.4.2
    """

    path: "Path" = betterproto.message_field(1)
    alias: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class AliasList(betterproto.Message):
    """
    AliasList specifies a list of aliases. It is used in a SubscribeRequest for
    a client to create a set of aliases that the target is to utilize.
    Reference: gNMI Specification Section 3.5.1.6
    """

    alias: List["Alias"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class SetRequest(betterproto.Message):
    """
    SetRequest is sent from a client to the target to update values in the data
    tree. Paths are either deleted by the client, or modified by means of being
    updated, or replaced. Where a replace is used, unspecified values are
    considered to be replaced, whereas when update is used the changes are
    considered to be incremental. The set of changes that are specified within
    a single SetRequest are considered to be a transaction. Reference: gNMI
    Specification Section 3.4.1
    """

    prefix: "Path" = betterproto.message_field(1)
    delete: List["Path"] = betterproto.message_field(2)
    replace: List["Update"] = betterproto.message_field(3)
    update: List["Update"] = betterproto.message_field(4)
    # Extension messages associated with the SetRequest. See the gNMI extension
    # specification for further definition.
    extension: List["_gnmi_ext__.Extension"] = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class SetResponse(betterproto.Message):
    """
    SetResponse is the response to a SetRequest, sent from the target to the
    client. It reports the result of the modifications to the data tree that
    were specified by the client. Errors for this RPC should be reported using
    the https://github.com/googleapis/googleapis/blob/master/google/rpc/status.
    proto message in the RPC return. The gnmi.Error message can be used to add
    additional details where required. Reference: gNMI Specification Section
    3.4.2
    """

    prefix: "Path" = betterproto.message_field(1)
    # A set of responses specifying the result of the operations specified in the
    # SetRequest.
    response: List["UpdateResult"] = betterproto.message_field(2)
    message: "Error" = betterproto.message_field(3)
    timestamp: int = betterproto.int64_field(4)
    # Extension messages associated with the SetResponse. See the gNMI extension
    # specification for further definition.
    extension: List["_gnmi_ext__.Extension"] = betterproto.message_field(5)

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.message:
            warnings.warn("SetResponse.message is deprecated", DeprecationWarning)


@dataclass(eq=False, repr=False)
class UpdateResult(betterproto.Message):
    """
    UpdateResult is used within the SetResponse message to communicate the
    result of an operation specified within a SetRequest message. Reference:
    gNMI Specification Section 3.4.2
    """

    # Deprecated timestamp for the UpdateResult, this field has been replaced by
    # the timestamp within the SetResponse message, since all mutations effected
    # by a set should be applied as a single transaction.
    timestamp: int = betterproto.int64_field(1)
    path: "Path" = betterproto.message_field(2)
    message: "Error" = betterproto.message_field(3)
    op: "UpdateResultOperation" = betterproto.enum_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.timestamp:
            warnings.warn("UpdateResult.timestamp is deprecated", DeprecationWarning)
        if self.message:
            warnings.warn("UpdateResult.message is deprecated", DeprecationWarning)


@dataclass(eq=False, repr=False)
class GetRequest(betterproto.Message):
    """
    GetRequest is sent when a client initiates a Get RPC. It is used to specify
    the set of data elements for which the target should return a snapshot of
    data. The use_models field specifies the set of schema modules that are to
    be used by the target - where use_models is not specified then the target
    must use all schema models that it has. Reference: gNMI Specification
    Section 3.3.1
    """

    prefix: "Path" = betterproto.message_field(1)
    path: List["Path"] = betterproto.message_field(2)
    type: "GetRequestDataType" = betterproto.enum_field(3)
    encoding: "Encoding" = betterproto.enum_field(5)
    use_models: List["ModelData"] = betterproto.message_field(6)
    # Extension messages associated with the GetRequest. See the gNMI extension
    # specification for further definition.
    extension: List["_gnmi_ext__.Extension"] = betterproto.message_field(7)


@dataclass(eq=False, repr=False)
class GetResponse(betterproto.Message):
    """
    GetResponse is used by the target to respond to a GetRequest from a client.
    The set of Notifications corresponds to the data values that are requested
    by the client in the GetRequest. Reference: gNMI Specification Section
    3.3.2
    """

    notification: List["Notification"] = betterproto.message_field(1)
    error: "Error" = betterproto.message_field(2)
    # Extension messages associated with the GetResponse. See the gNMI extension
    # specification for further definition.
    extension: List["_gnmi_ext__.Extension"] = betterproto.message_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.error:
            warnings.warn("GetResponse.error is deprecated", DeprecationWarning)


@dataclass(eq=False, repr=False)
class CapabilityRequest(betterproto.Message):
    """
    CapabilityRequest is sent by the client in the Capabilities RPC to request
    that the target reports its capabilities. Reference: gNMI Specification
    Section 3.2.1
    """

    # Extension messages associated with the CapabilityRequest. See the gNMI
    # extension specification for further definition.
    extension: List["_gnmi_ext__.Extension"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CapabilityResponse(betterproto.Message):
    """
    CapabilityResponse is used by the target to report its capabilities to the
    client within the Capabilities RPC. Reference: gNMI Specification Section
    3.2.2
    """

    supported_models: List["ModelData"] = betterproto.message_field(1)
    supported_encodings: List["Encoding"] = betterproto.enum_field(2)
    g_nmi_version: str = betterproto.string_field(3)
    # Extension messages associated with the CapabilityResponse. See the gNMI
    # extension specification for further definition.
    extension: List["_gnmi_ext__.Extension"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class ModelData(betterproto.Message):
    """
    ModelData is used to describe a set of schema modules. It can be used in a
    CapabilityResponse where a target reports the set of modules that it
    supports, and within the SubscribeRequest and GetRequest messages to
    specify the set of models from which data tree elements should be reported.
    Reference: gNMI Specification Section 3.2.3
    """

    name: str = betterproto.string_field(1)
    organization: str = betterproto.string_field(2)
    version: str = betterproto.string_field(3)


class GNmiStub(betterproto.ServiceStub):
    async def capabilities(
        self, *, extension: Optional[List["_gnmi_ext__.Extension"]] = None
    ) -> "CapabilityResponse":
        extension = extension or []

        request = CapabilityRequest()
        if extension is not None:
            request.extension = extension

        return await self._unary_unary(
            "/gnmi.gNMI/Capabilities", request, CapabilityResponse
        )

    async def get(
        self,
        *,
        prefix: "Path" = None,
        path: Optional[List["Path"]] = None,
        type: "GetRequestDataType" = None,
        encoding: "Encoding" = None,
        use_models: Optional[List["ModelData"]] = None,
        extension: Optional[List["_gnmi_ext__.Extension"]] = None,
    ) -> "GetResponse":
        path = path or []
        use_models = use_models or []
        extension = extension or []

        request = GetRequest()
        if prefix is not None:
            request.prefix = prefix
        if path is not None:
            request.path = path
        request.type = type
        request.encoding = encoding
        if use_models is not None:
            request.use_models = use_models
        if extension is not None:
            request.extension = extension

        return await self._unary_unary("/gnmi.gNMI/Get", request, GetResponse)

    async def set(
        self,
        *,
        prefix: "Path" = None,
        delete: Optional[List["Path"]] = None,
        replace: Optional[List["Update"]] = None,
        update: Optional[List["Update"]] = None,
        extension: Optional[List["_gnmi_ext__.Extension"]] = None,
    ) -> "SetResponse":
        delete = delete or []
        replace = replace or []
        update = update or []
        extension = extension or []

        request = SetRequest()
        if prefix is not None:
            request.prefix = prefix
        if delete is not None:
            request.delete = delete
        if replace is not None:
            request.replace = replace
        if update is not None:
            request.update = update
        if extension is not None:
            request.extension = extension

        return await self._unary_unary("/gnmi.gNMI/Set", request, SetResponse)

    async def subscribe(
        self,
        request_iterator: Union[
            AsyncIterable["SubscribeRequest"], Iterable["SubscribeRequest"]
        ],
    ) -> AsyncIterator["SubscribeResponse"]:

        async for response in self._stream_stream(
            "/gnmi.gNMI/Subscribe",
            request_iterator,
            SubscribeRequest,
            SubscribeResponse,
        ):
            yield response


class GNmiBase(ServiceBase):
    async def capabilities(
        self, extension: Optional[List["_gnmi_ext__.Extension"]]
    ) -> "CapabilityResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get(
        self,
        prefix: "Path",
        path: Optional[List["Path"]],
        type: "GetRequestDataType",
        encoding: "Encoding",
        use_models: Optional[List["ModelData"]],
        extension: Optional[List["_gnmi_ext__.Extension"]],
    ) -> "GetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set(
        self,
        prefix: "Path",
        delete: Optional[List["Path"]],
        replace: Optional[List["Update"]],
        update: Optional[List["Update"]],
        extension: Optional[List["_gnmi_ext__.Extension"]],
    ) -> "SetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def subscribe(
        self, request_iterator: AsyncIterator["SubscribeRequest"]
    ) -> AsyncIterator["SubscribeResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_capabilities(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "extension": request.extension,
        }

        response = await self.capabilities(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "prefix": request.prefix,
            "path": request.path,
            "type": request.type,
            "encoding": request.encoding,
            "use_models": request.use_models,
            "extension": request.extension,
        }

        response = await self.get(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_set(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "prefix": request.prefix,
            "delete": request.delete,
            "replace": request.replace,
            "update": request.update,
            "extension": request.extension,
        }

        response = await self.set(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_subscribe(self, stream: grpclib.server.Stream) -> None:
        request_kwargs = {"request_iterator": stream.__aiter__()}

        await self._call_rpc_handler_server_stream(
            self.subscribe,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/gnmi.gNMI/Capabilities": grpclib.const.Handler(
                self.__rpc_capabilities,
                grpclib.const.Cardinality.UNARY_UNARY,
                CapabilityRequest,
                CapabilityResponse,
            ),
            "/gnmi.gNMI/Get": grpclib.const.Handler(
                self.__rpc_get,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetRequest,
                GetResponse,
            ),
            "/gnmi.gNMI/Set": grpclib.const.Handler(
                self.__rpc_set,
                grpclib.const.Cardinality.UNARY_UNARY,
                SetRequest,
                SetResponse,
            ),
            "/gnmi.gNMI/Subscribe": grpclib.const.Handler(
                self.__rpc_subscribe,
                grpclib.const.Cardinality.STREAM_STREAM,
                SubscribeRequest,
                SubscribeResponse,
            ),
        }


from .. import gnmi_ext as _gnmi_ext__
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/mho/mho.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict, List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class MhoParamType(betterproto.Enum):
    """MhoParamType is enumeration type of MHO parameters"""

    ALL = 0
    A3OFFSET = 1
    HYSTERESIS = 2
    TIMETOTRIGGER = 3


@dataclass(eq=False, repr=False)
class GetMhoParamRequest(betterproto.Message):
    # hoParamType is a type of handover parameter
    ho_param_type: "MhoParamType" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class GetMhoParamResponse(betterproto.Message):
    # hoParamType is a type of handover parameter
    ho_param_type: "MhoParamType" = betterproto.enum_field(1)
    # A3-Offset value
    a3_offset: int = betterproto.int32_field(2)
    # Hysteresis value
    hysteresis: int = betterproto.int32_field(3)
    # Time-to-Trigger value
    time_to_trigger: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class SetMhoParamRequest(betterproto.Message):
    # hoParamType is a type of handover parameter
    ho_param_type: "MhoParamType" = betterproto.enum_field(1)
    # A3-Offset value
    a3_offset: int = betterproto.int32_field(2)
    # Hysteresis value
    hysteresis: int = betterproto.int32_field(3)
    # Time-to-Trigger value
    time_to_trigger: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class SetMhoParamResponse(betterproto.Message):
    # success is a result whether MHO param is set successfully or not
    success: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class GetRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class UeList(betterproto.Message):
    ues: List["Ue"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CellList(betterproto.Message):
    cells: List["Cell"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Ue(betterproto.Message):
    ue_id: str = betterproto.string_field(1)
    rrc_state: str = betterproto.string_field(2)
    cgi: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class Cell(betterproto.Message):
    cgi: str = betterproto.string_field(1)
    num_ues: int = betterproto.int64_field(2)
    cumulative_handovers_in: int = betterproto.int64_field(4)
    cumulative_handovers_out: int = betterproto.int64_field(5)


class MhoStub(betterproto.ServiceStub):
    async def get_mho_params(
        self, *, ho_param_type: "MhoParamType" = None
    ) -> "GetMhoParamResponse":

        request = GetMhoParamRequest()
        request.ho_param_type = ho_param_type

        return await self._unary_unary(
            "/onos.mho.mho/GetMhoParams", request, GetMhoParamResponse
        )

    async def set_mho_params(
        self,
        *,
        ho_param_type: "MhoParamType" = None,
        a3_offset: int = 0,
        hysteresis: int = 0,
        time_to_trigger: int = 0,
    ) -> "SetMhoParamResponse":

        request = SetMhoParamRequest()
        request.ho_param_type = ho_param_type
        request.a3_offset = a3_offset
        request.hysteresis = hysteresis
        request.time_to_trigger = time_to_trigger

        return await self._unary_unary(
            "/onos.mho.mho/SetMhoParams", request, SetMhoParamResponse
        )

    async def get_ues(self) -> "UeList":

        request = GetRequest()

        return await self._unary_unary("/onos.mho.mho/GetUes", request, UeList)

    async def get_cells(self) -> "CellList":

        request = GetRequest()

        return await self._unary_unary("/onos.mho.mho/GetCells", request, CellList)


class MhoBase(ServiceBase):
    async def get_mho_params(
        self, ho_param_type: "MhoParamType"
    ) -> "GetMhoParamResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set_mho_params(
        self,
        ho_param_type: "MhoParamType",
        a3_offset: int,
        hysteresis: int,
        time_to_trigger: int,
    ) -> "SetMhoParamResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_ues(self) -> "UeList":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_cells(self) -> "CellList":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_mho_params(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "ho_param_type": request.ho_param_type,
        }

        response = await self.get_mho_params(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_set_mho_params(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "ho_param_type": request.ho_param_type,
            "a3_offset": request.a3_offset,
            "hysteresis": request.hysteresis,
            "time_to_trigger": request.time_to_trigger,
        }

        response = await self.set_mho_params(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get_ues(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_ues(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get_cells(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_cells(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.mho.mho/GetMhoParams": grpclib.const.Handler(
                self.__rpc_get_mho_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetMhoParamRequest,
                GetMhoParamResponse,
            ),
            "/onos.mho.mho/SetMhoParams": grpclib.const.Handler(
                self.__rpc_set_mho_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                SetMhoParamRequest,
                SetMhoParamResponse,
            ),
            "/onos.mho.mho/GetUes": grpclib.const.Handler(
                self.__rpc_get_ues,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetRequest,
                UeList,
            ),
            "/onos.mho.mho/GetCells": grpclib.const.Handler(
                self.__rpc_get_cells,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetRequest,
                CellList,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/uenib/ran.proto, onos/uenib/uenib.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, Dict, List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class UeIdType(betterproto.Enum):
    UE_ID_TYPE_CU_UE_F1_AP_ID = 0
    UE_ID_TYPE_DU_UE_F1_AP_ID = 1
    UE_ID_TYPE_RAN_UE_NGAP_ID = 2
    UE_ID_TYPE_AMF_UE_NGAP_ID = 3
    UE_ID_TYPE_ENB_UE_S1_AP_ID = 4


class RsmSchedulerType(betterproto.Enum):
    SCHEDULER_TYPE_ROUND_ROBIN = 0
    SCHEDULER_TYPE_PROPORTIONALLY_FAIR = 1
    SCHEDULER_TYPE_QOS_BASED = 2


class RsmSliceType(betterproto.Enum):
    SLICE_TYPE_DL_SLICE = 0
    SLICE_TYPE_UL_SLICE = 1


class EventType(betterproto.Enum):
    """EventType is a UE operation event type"""

    NONE = 0
    ADDED = 1
    UPDATED = 2
    REMOVED = 3


@dataclass(eq=False, repr=False)
class CellConnection(betterproto.Message):
    """CellConnection represents UE cell connection."""

    id: str = betterproto.string_field(1)
    signal_strength: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class CellInfo(betterproto.Message):
    """CellInfo provides data on serving cell and candidate cells."""

    serving_cell: "CellConnection" = betterproto.message_field(1)
    candidate_cells: List["CellConnection"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class DuUeF1ApId(betterproto.Message):
    value: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class CuUeF1ApId(betterproto.Message):
    value: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class RanUeNgapId(betterproto.Message):
    value: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class EnbUeS1ApId(betterproto.Message):
    value: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class AmfUeNgapId(betterproto.Message):
    value: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class UeIdentity(betterproto.Message):
    du_ue_f1_ap_id: "DuUeF1ApId" = betterproto.message_field(1)
    cu_ue_f1_ap_id: "CuUeF1ApId" = betterproto.message_field(2)
    ran_ue_ngap_id: "RanUeNgapId" = betterproto.message_field(3)
    enb_ue_s1_ap_id: "EnbUeS1ApId" = betterproto.message_field(4)
    amf_ue_ngap_id: "AmfUeNgapId" = betterproto.message_field(5)
    preferred_id_type: "UeIdType" = betterproto.enum_field(6)


@dataclass(eq=False, repr=False)
class BearerId(betterproto.Message):
    drb_id: "DrbId" = betterproto.message_field(1, group="bearer_id")


@dataclass(eq=False, repr=False)
class DrbId(betterproto.Message):
    four_gdrb_id: "FourGDrbId" = betterproto.message_field(1, group="drb_id")
    five_gdrb_id: "FiveGDrbId" = betterproto.message_field(2, group="drb_id")


@dataclass(eq=False, repr=False)
class FiveGDrbId(betterproto.Message):
    value: int = betterproto.int32_field(1)
    qfi: "Qfi" = betterproto.message_field(2)
    flows_map_to_drb: List["QoSflowLevelParameters"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class Qfi(betterproto.Message):
    value: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class QoSflowLevelParameters(betterproto.Message):
    dynamic_five_qi: "DynamicFiveQi" = betterproto.message_field(
        1, group="qos_flow_level_parameters"
    )
    non_dynamic_five_qi: "NonDynamicFiveQi" = betterproto.message_field(
        2, group="qos_flow_level_parameters"
    )


@dataclass(eq=False, repr=False)
class DynamicFiveQi(betterproto.Message):
    priority_level: int = betterproto.int32_field(1)
    packet_delay_budge: int = betterproto.int32_field(2)
    packet_error_rate: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class NonDynamicFiveQi(betterproto.Message):
    five_qi: "FiveQi" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class FiveQi(betterproto.Message):
    value: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class FourGDrbId(betterproto.Message):
    value: int = betterproto.int32_field(1)
    qci: "Qci" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Qci(betterproto.Message):
    value: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class RsmUeInfo(betterproto.Message):
    global_ue_id: str = betterproto.string_field(1)
    ue_id_list: "UeIdentity" = betterproto.message_field(2)
    bearer_id_list: List["BearerId"] = betterproto.message_field(3)
    cell_global_id: str = betterproto.string_field(4)
    cu_e2_node_id: str = betterproto.string_field(5)
    du_e2_node_id: str = betterproto.string_field(6)
    slice_list: List["SliceInfo"] = betterproto.message_field(7)


@dataclass(eq=False, repr=False)
class SliceInfo(betterproto.Message):
    du_e2_node_id: str = betterproto.string_field(1)
    cu_e2_node_id: str = betterproto.string_field(2)
    id: str = betterproto.string_field(3)
    slice_desc: str = betterproto.string_field(4)
    slice_parameters: "RsmSliceParameters" = betterproto.message_field(5)
    slice_type: "RsmSliceType" = betterproto.enum_field(6)
    drb_id: "DrbId" = betterproto.message_field(7)


@dataclass(eq=False, repr=False)
class RsmSliceParameters(betterproto.Message):
    scheduler_type: "RsmSchedulerType" = betterproto.enum_field(1)
    weight: int = betterproto.int32_field(2)
    qos_level: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class Event(betterproto.Message):
    """Event is a record of an operation on a UE"""

    type: "EventType" = betterproto.enum_field(1)
    ue: "Ue" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CreateUeRequest(betterproto.Message):
    ue: "Ue" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CreateUeResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetUeRequest(betterproto.Message):
    id: str = betterproto.string_field(1)
    aspect_types: List[str] = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class GetUeResponse(betterproto.Message):
    ue: "Ue" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class UpdateUeRequest(betterproto.Message):
    ue: "Ue" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class UpdateUeResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class DeleteUeRequest(betterproto.Message):
    id: str = betterproto.string_field(1)
    aspect_types: List[str] = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class DeleteUeResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ListUeRequest(betterproto.Message):
    aspect_types: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ListUeResponse(betterproto.Message):
    ue: "Ue" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class WatchUeRequest(betterproto.Message):
    noreplay: bool = betterproto.bool_field(2)
    aspect_types: List[str] = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class WatchUeResponse(betterproto.Message):
    event: "Event" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Ue(betterproto.Message):
    """UE entity is merely an ID and a map of arbitrary aspects."""

    id: str = betterproto.string_field(1)
    # Map of aspects as typed values
    aspects: Dict[str, "betterproto_lib_google_protobuf.Any"] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


class UeServiceStub(betterproto.ServiceStub):
    async def create_ue(self) -> "CreateUeResponse":

        request = CreateUeRequest()

        return await self._unary_unary(
            "/onos.uenib.UEService/CreateUE", request, CreateUeResponse
        )

    async def get_ue(self) -> "GetUeResponse":

        request = GetUeRequest()

        return await self._unary_unary(
            "/onos.uenib.UEService/GetUE", request, GetUeResponse
        )

    async def update_ue(self) -> "UpdateUeResponse":

        request = UpdateUeRequest()

        return await self._unary_unary(
            "/onos.uenib.UEService/UpdateUE", request, UpdateUeResponse
        )

    async def delete_ue(self) -> "DeleteUeResponse":

        request = DeleteUeRequest()

        return await self._unary_unary(
            "/onos.uenib.UEService/DeleteUE", request, DeleteUeResponse
        )

    async def list_u_es(self) -> AsyncIterator["ListUeResponse"]:

        request = ListUeRequest()

        async for response in self._unary_stream(
            "/onos.uenib.UEService/ListUEs",
            request,
            ListUeResponse,
        ):
            yield response

    async def watch_u_es(self) -> AsyncIterator["WatchUeResponse"]:

        request = WatchUeRequest()

        async for response in self._unary_stream(
            "/onos.uenib.UEService/WatchUEs",
            request,
            WatchUeResponse,
        ):
            yield response


class UeServiceBase(ServiceBase):
    async def create_ue(self) -> "CreateUeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_ue(self) -> "GetUeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def update_ue(self) -> "UpdateUeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete_ue(self) -> "DeleteUeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_u_es(self) -> AsyncIterator["ListUeResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch_u_es(self) -> AsyncIterator["WatchUeResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_create_ue(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.create_ue(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get_ue(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_ue(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_update_ue(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.update_ue(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete_ue(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.delete_ue(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list_u_es(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        await self._call_rpc_handler_server_stream(
            self.list_u_es,
            stream,
            request_kwargs,
        )

    async def __rpc_watch_u_es(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        await self._call_rpc_handler_server_stream(
            self.watch_u_es,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.uenib.UEService/CreateUE": grpclib.const.Handler(
                self.__rpc_create_ue,
                grpclib.const.Cardinality.UNARY_UNARY,
                CreateUeRequest,
                CreateUeResponse,
            ),
            "/onos.uenib.UEService/GetUE": grpclib.const.Handler(
                self.__rpc_get_ue,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetUeRequest,
                GetUeResponse,
            ),
            "/onos.uenib.UEService/UpdateUE": grpclib.const.Handler(
                self.__rpc_update_ue,
                grpclib.const.Cardinality.UNARY_UNARY,
                UpdateUeRequest,
                UpdateUeResponse,
            ),
            "/onos.uenib.UEService/DeleteUE": grpclib.const.Handler(
                self.__rpc_delete_ue,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteUeRequest,
                DeleteUeResponse,
            ),
            "/onos.uenib.UEService/ListUEs": grpclib.const.Handler(
                self.__rpc_list_u_es,
                grpclib.const.Cardinality.UNARY_STREAM,
                ListUeRequest,
                ListUeResponse,
            ),
            "/onos.uenib.UEService/WatchUEs": grpclib.const.Handler(
                self.__rpc_watch_u_es,
                grpclib.const.Cardinality.UNARY_STREAM,
                WatchUeRequest,
                WatchUeResponse,
            ),
        }


import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/perf/perf.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterable, AsyncIterator, Dict, Iterable, Union

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


@dataclass(eq=False, repr=False)
class Data(betterproto.Message):
    length: int = betterproto.uint32_field(1)
    data: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class PingRequest(betterproto.Message):
    payload: "Data" = betterproto.message_field(1)
    timestamp: int = betterproto.uint64_field(2)
    repeat_count: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class PingResponse(betterproto.Message):
    payload: "Data" = betterproto.message_field(1)
    timestamp: int = betterproto.uint64_field(2)


class PerfServiceStub(betterproto.ServiceStub):
    async def ping(
        self, *, payload: "Data" = None, timestamp: int = 0, repeat_count: int = 0
    ) -> "PingResponse":

        request = PingRequest()
        if payload is not None:
            request.payload = payload
        request.timestamp = timestamp
        request.repeat_count = repeat_count

        return await self._unary_unary(
            "/onos.perf.PerfService/Ping", request, PingResponse
        )

    async def ping_stream(
        self,
        request_iterator: Union[AsyncIterable["PingRequest"], Iterable["PingRequest"]],
    ) -> AsyncIterator["PingResponse"]:

        async for response in self._stream_stream(
            "/onos.perf.PerfService/PingStream",
            request_iterator,
            PingRequest,
            PingResponse,
        ):
            yield response


class PerfServiceBase(ServiceBase):
    async def ping(
        self, payload: "Data", timestamp: int, repeat_count: int
    ) -> "PingResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def ping_stream(
        self, request_iterator: AsyncIterator["PingRequest"]
    ) -> AsyncIterator["PingResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_ping(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "payload": request.payload,
            "timestamp": request.timestamp,
            "repeat_count": request.repeat_count,
        }

        response = await self.ping(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_ping_stream(self, stream: grpclib.server.Stream) -> None:
        request_kwargs = {"request_iterator": stream.__aiter__()}

        await self._call_rpc_handler_server_stream(
            self.ping_stream,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.perf.PerfService/Ping": grpclib.const.Handler(
                self.__rpc_ping,
                grpclib.const.Cardinality.UNARY_UNARY,
                PingRequest,
                PingResponse,
            ),
            "/onos.perf.PerfService/PingStream": grpclib.const.Handler(
                self.__rpc_ping_stream,
                grpclib.const.Cardinality.STREAM_STREAM,
                PingRequest,
                PingResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/configmodel/registry.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict, List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class GetStateMode(betterproto.Enum):
    NONE = 0
    OP_STATE = 1
    EXPLICIT_RO_PATHS = 2
    EXPLICIT_RO_PATHS_EXPAND_WILDCARDS = 3


@dataclass(eq=False, repr=False)
class ConfigModel(betterproto.Message):
    name: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)
    modules: List["ConfigModule"] = betterproto.message_field(3)
    files: Dict[str, str] = betterproto.map_field(
        4, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    get_state_mode: "GetStateMode" = betterproto.enum_field(5)


@dataclass(eq=False, repr=False)
class ConfigModule(betterproto.Message):
    name: str = betterproto.string_field(1)
    file: str = betterproto.string_field(2)
    revision: str = betterproto.string_field(3)
    organization: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class GetModelRequest(betterproto.Message):
    name: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class GetModelResponse(betterproto.Message):
    model: "ConfigModel" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ListModelsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ListModelsResponse(betterproto.Message):
    models: List["ConfigModel"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class PushModelRequest(betterproto.Message):
    model: "ConfigModel" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class PushModelResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class DeleteModelRequest(betterproto.Message):
    name: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class DeleteModelResponse(betterproto.Message):
    pass


class ConfigModelRegistryServiceStub(betterproto.ServiceStub):
    async def get_model(
        self, *, name: str = "", version: str = ""
    ) -> "GetModelResponse":

        request = GetModelRequest()
        request.name = name
        request.version = version

        return await self._unary_unary(
            "/onos.configmodel.ConfigModelRegistryService/GetModel",
            request,
            GetModelResponse,
        )

    async def list_models(self) -> "ListModelsResponse":

        request = ListModelsRequest()

        return await self._unary_unary(
            "/onos.configmodel.ConfigModelRegistryService/ListModels",
            request,
            ListModelsResponse,
        )

    async def push_model(self, *, model: "ConfigModel" = None) -> "PushModelResponse":

        request = PushModelRequest()
        if model is not None:
            request.model = model

        return await self._unary_unary(
            "/onos.configmodel.ConfigModelRegistryService/PushModel",
            request,
            PushModelResponse,
        )

    async def delete_model(
        self, *, name: str = "", version: str = ""
    ) -> "DeleteModelResponse":

        request = DeleteModelRequest()
        request.name = name
        request.version = version

        return await self._unary_unary(
            "/onos.configmodel.ConfigModelRegistryService/DeleteModel",
            request,
            DeleteModelResponse,
        )


class ConfigModelRegistryServiceBase(ServiceBase):
    async def get_model(self, name: str, version: str) -> "GetModelResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_models(self) -> "ListModelsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def push_model(self, model: "ConfigModel") -> "PushModelResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete_model(self, name: str, version: str) -> "DeleteModelResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_model(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "name": request.name,
            "version": request.version,
        }

        response = await self.get_model(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list_models(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.list_models(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_push_model(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "model": request.model,
        }

        response = await self.push_model(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete_model(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "name": request.name,
            "version": request.version,
        }

        response = await self.delete_model(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.configmodel.ConfigModelRegistryService/GetModel": grpclib.const.Handler(
                self.__rpc_get_model,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetModelRequest,
                GetModelResponse,
            ),
            "/onos.configmodel.ConfigModelRegistryService/ListModels": grpclib.const.Handler(
                self.__rpc_list_models,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListModelsRequest,
                ListModelsResponse,
            ),
            "/onos.configmodel.ConfigModelRegistryService/PushModel": grpclib.const.Handler(
                self.__rpc_push_model,
                grpclib.const.Cardinality.UNARY_UNARY,
                PushModelRequest,
                PushModelResponse,
            ),
            "/onos.configmodel.ConfigModelRegistryService/DeleteModel": grpclib.const.Handler(
                self.__rpc_delete_model,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteModelRequest,
                DeleteModelResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: gogoproto/gogo.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase



#!/usr/bin/env python3

# Copyright 2021-present Open Networking Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages


PACKAGES = find_packages("python")


def readme():
    with open("./README.md") as f:
        return f.read()


def version():
    with open("VERSION") as f:
        return f.read()


setup(
    name="onos-api",
    version=version(),
    description="ONOS Python API",
    long_description=readme(),
    long_description_content_type="text/markdown",
    author="Open Networking Foundation (ONF) and Partners",
    author_email="support@opennetworking.org",
    packages=[pkg.replace("onos", "onos_api") for pkg in PACKAGES],
    package_dir={
        pkg.replace("onos", "onos_api"): f"python/{pkg}".replace(".", "/")
        for pkg in PACKAGES
    },
    package_data={"onos_api": ["py.typed"]},
    keywords=["onos-api", "python", "protobuf", "gnmi"],
    license="Apache License v2.0",
    url="https://github.com/onosproject/onos-api/",
    download_url="https://github.com/onosproject/onos-api/",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Utilities",
        "License :: OSI Approved :: Apache Software License",
        "Typing :: Typed",
    ],
    install_requires=[
        "betterproto>=2.0.0b3,<3",
        "gnmi-proto>=0.1.0a4,<1",
    ],
    python_requires=">=3.6",
    setup_requires=["setuptools>=41.1.0"],
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/kpimon/kpimon.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, Dict, List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


@dataclass(eq=False, repr=False)
class GetRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetResponse(betterproto.Message):
    measurements: Dict[str, "MeasurementItems"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass(eq=False, repr=False)
class MeasurementItems(betterproto.Message):
    measurement_items: List["MeasurementItem"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class MeasurementItem(betterproto.Message):
    measurement_records: List["MeasurementRecord"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class IntegerValue(betterproto.Message):
    value: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class RealValue(betterproto.Message):
    value: float = betterproto.double_field(1)


@dataclass(eq=False, repr=False)
class NoValue(betterproto.Message):
    value: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class MeasurementRecord(betterproto.Message):
    timestamp: int = betterproto.uint64_field(2)
    measurement_name: str = betterproto.string_field(3)
    measurement_value: "betterproto_lib_google_protobuf.Any" = (
        betterproto.message_field(4)
    )


class KpimonStub(betterproto.ServiceStub):
    async def list_measurements(self) -> "GetResponse":

        request = GetRequest()

        return await self._unary_unary(
            "/onos.kpimon.Kpimon/ListMeasurements", request, GetResponse
        )

    async def watch_measurements(self) -> AsyncIterator["GetResponse"]:

        request = GetRequest()

        async for response in self._unary_stream(
            "/onos.kpimon.Kpimon/WatchMeasurements",
            request,
            GetResponse,
        ):
            yield response


class KpimonBase(ServiceBase):
    async def list_measurements(self) -> "GetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch_measurements(self) -> AsyncIterator["GetResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_list_measurements(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.list_measurements(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_watch_measurements(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        await self._call_rpc_handler_server_stream(
            self.watch_measurements,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.kpimon.Kpimon/ListMeasurements": grpclib.const.Handler(
                self.__rpc_list_measurements,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetRequest,
                GetResponse,
            ),
            "/onos.kpimon.Kpimon/WatchMeasurements": grpclib.const.Handler(
                self.__rpc_watch_measurements,
                grpclib.const.Cardinality.UNARY_STREAM,
                GetRequest,
                GetResponse,
            ),
        }


import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/ransim/metrics/metrics.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, Dict, List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class EventType(betterproto.Enum):
    """Change event type"""

    # NONE indicates unknown event type
    NONE = 0
    # UPDATED indicates a metric value was set (updated)
    UPDATED = 1
    # DELETED indicates a metric was deleted
    DELETED = 2


@dataclass(eq=False, repr=False)
class Metric(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)
    key: str = betterproto.string_field(2)
    value: str = betterproto.string_field(3)
    type: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class ListRequest(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class ListResponse(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)
    metrics: List["Metric"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class SetRequest(betterproto.Message):
    metric: "Metric" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class SetResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetRequest(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)
    name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class GetResponse(betterproto.Message):
    metric: "Metric" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class DeleteRequest(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)
    name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class DeleteResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class DeleteAllRequest(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class DeleteAllResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class WatchRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class WatchResponse(betterproto.Message):
    metric: "Metric" = betterproto.message_field(1)
    type: "EventType" = betterproto.enum_field(2)


class MetricsServiceStub(betterproto.ServiceStub):
    async def list(self, *, entityid: int = 0) -> "ListResponse":

        request = ListRequest()
        request.entityid = entityid

        return await self._unary_unary(
            "/onos.ransim.metrics.MetricsService/List", request, ListResponse
        )

    async def set(self, *, metric: "Metric" = None) -> "SetResponse":

        request = SetRequest()
        if metric is not None:
            request.metric = metric

        return await self._unary_unary(
            "/onos.ransim.metrics.MetricsService/Set", request, SetResponse
        )

    async def get(self, *, entityid: int = 0, name: str = "") -> "GetResponse":

        request = GetRequest()
        request.entityid = entityid
        request.name = name

        return await self._unary_unary(
            "/onos.ransim.metrics.MetricsService/Get", request, GetResponse
        )

    async def delete(self, *, entityid: int = 0, name: str = "") -> "DeleteResponse":

        request = DeleteRequest()
        request.entityid = entityid
        request.name = name

        return await self._unary_unary(
            "/onos.ransim.metrics.MetricsService/Delete", request, DeleteResponse
        )

    async def delete_all(self, *, entityid: int = 0) -> "DeleteAllResponse":

        request = DeleteAllRequest()
        request.entityid = entityid

        return await self._unary_unary(
            "/onos.ransim.metrics.MetricsService/DeleteAll", request, DeleteAllResponse
        )

    async def watch(self) -> AsyncIterator["WatchResponse"]:

        request = WatchRequest()

        async for response in self._unary_stream(
            "/onos.ransim.metrics.MetricsService/Watch",
            request,
            WatchResponse,
        ):
            yield response


class MetricsServiceBase(ServiceBase):
    async def list(self, entityid: int) -> "ListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set(self, metric: "Metric") -> "SetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get(self, entityid: int, name: str) -> "GetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete(self, entityid: int, name: str) -> "DeleteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete_all(self, entityid: int) -> "DeleteAllResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch(self) -> AsyncIterator["WatchResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_list(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "entityid": request.entityid,
        }

        response = await self.list(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_set(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "metric": request.metric,
        }

        response = await self.set(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "entityid": request.entityid,
            "name": request.name,
        }

        response = await self.get(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "entityid": request.entityid,
            "name": request.name,
        }

        response = await self.delete(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete_all(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "entityid": request.entityid,
        }

        response = await self.delete_all(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_watch(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        await self._call_rpc_handler_server_stream(
            self.watch,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.ransim.metrics.MetricsService/List": grpclib.const.Handler(
                self.__rpc_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListRequest,
                ListResponse,
            ),
            "/onos.ransim.metrics.MetricsService/Set": grpclib.const.Handler(
                self.__rpc_set,
                grpclib.const.Cardinality.UNARY_UNARY,
                SetRequest,
                SetResponse,
            ),
            "/onos.ransim.metrics.MetricsService/Get": grpclib.const.Handler(
                self.__rpc_get,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetRequest,
                GetResponse,
            ),
            "/onos.ransim.metrics.MetricsService/Delete": grpclib.const.Handler(
                self.__rpc_delete,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteRequest,
                DeleteResponse,
            ),
            "/onos.ransim.metrics.MetricsService/DeleteAll": grpclib.const.Handler(
                self.__rpc_delete_all,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteAllRequest,
                DeleteAllResponse,
            ),
            "/onos.ransim.metrics.MetricsService/Watch": grpclib.const.Handler(
                self.__rpc_watch,
                grpclib.const.Cardinality.UNARY_STREAM,
                WatchRequest,
                WatchResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/pci/pci.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict, List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class CellType(betterproto.Enum):
    FEMTO = 0
    ENTERPRISE = 1
    OUTDOOR_SMALL = 2
    MACRO = 3


@dataclass(eq=False, repr=False)
class GetConflictsRequest(betterproto.Message):
    """if cell id is not specified, will return all cells with conflicts"""

    cell_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class GetConflictsResponse(betterproto.Message):
    cells: List["PciCell"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class GetResolvedConflictsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetResolvedConflictsResponse(betterproto.Message):
    """returns all the resolved conflicts in the store"""

    cells: List["CellResolution"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CellResolution(betterproto.Message):
    id: int = betterproto.uint64_field(1)
    resolved_pci: int = betterproto.uint32_field(2)
    original_pci: int = betterproto.uint32_field(3)
    resolved_conflicts: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class GetCellRequest(betterproto.Message):
    """cell id required"""

    cell_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class GetCellResponse(betterproto.Message):
    cell: "PciCell" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class GetCellsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetCellsResponse(betterproto.Message):
    cells: List["PciCell"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class PciCell(betterproto.Message):
    id: int = betterproto.uint64_field(1)
    node_id: str = betterproto.string_field(2)
    dlearfcn: int = betterproto.uint32_field(3)
    cell_type: "CellType" = betterproto.enum_field(4)
    pci: int = betterproto.uint32_field(5)
    pci_pool: List["PciRange"] = betterproto.message_field(6)
    neighbor_ids: List[int] = betterproto.uint64_field(7)


@dataclass(eq=False, repr=False)
class PciRange(betterproto.Message):
    min: int = betterproto.uint32_field(1)
    max: int = betterproto.uint32_field(2)


class PciStub(betterproto.ServiceStub):
    async def get_conflicts(self, *, cell_id: int = 0) -> "GetConflictsResponse":

        request = GetConflictsRequest()
        request.cell_id = cell_id

        return await self._unary_unary(
            "/onos.pci.Pci/GetConflicts", request, GetConflictsResponse
        )

    async def get_resolved_conflicts(self) -> "GetResolvedConflictsResponse":

        request = GetResolvedConflictsRequest()

        return await self._unary_unary(
            "/onos.pci.Pci/GetResolvedConflicts", request, GetResolvedConflictsResponse
        )

    async def get_cell(self, *, cell_id: int = 0) -> "GetCellResponse":

        request = GetCellRequest()
        request.cell_id = cell_id

        return await self._unary_unary(
            "/onos.pci.Pci/GetCell", request, GetCellResponse
        )

    async def get_cells(self) -> "GetCellsResponse":

        request = GetCellsRequest()

        return await self._unary_unary(
            "/onos.pci.Pci/GetCells", request, GetCellsResponse
        )


class PciBase(ServiceBase):
    async def get_conflicts(self, cell_id: int) -> "GetConflictsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_resolved_conflicts(self) -> "GetResolvedConflictsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_cell(self, cell_id: int) -> "GetCellResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_cells(self) -> "GetCellsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_conflicts(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "cell_id": request.cell_id,
        }

        response = await self.get_conflicts(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get_resolved_conflicts(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_resolved_conflicts(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get_cell(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "cell_id": request.cell_id,
        }

        response = await self.get_cell(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get_cells(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_cells(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.pci.Pci/GetConflicts": grpclib.const.Handler(
                self.__rpc_get_conflicts,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetConflictsRequest,
                GetConflictsResponse,
            ),
            "/onos.pci.Pci/GetResolvedConflicts": grpclib.const.Handler(
                self.__rpc_get_resolved_conflicts,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetResolvedConflictsRequest,
                GetResolvedConflictsResponse,
            ),
            "/onos.pci.Pci/GetCell": grpclib.const.Handler(
                self.__rpc_get_cell,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetCellRequest,
                GetCellResponse,
            ),
            "/onos.pci.Pci/GetCells": grpclib.const.Handler(
                self.__rpc_get_cells,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetCellsRequest,
                GetCellsResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/e2t/e2/v1beta2/e2.proto, onos/e2t/e2/v1beta2/control.proto, onos/e2t/e2/v1beta2/subscription.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, List, Optional

import betterproto
import grpclib


class Encoding(betterproto.Enum):
    PROTO = 0
    ASN1_PER = 1
    ASN1_XER = 2


class ErrorCauseMiscType(betterproto.Enum):
    UNSPECIFIED = 0
    CONTROL_PROCESSING_OVERLOAD = 1
    HARDWARE_FAILURE = 2
    OM_INTERVENTION = 3


class ErrorCauseProtocolType(betterproto.Enum):
    UNSPECIFIED = 0
    TRANSFER_SYNTAX_ERROR = 1
    ABSTRACT_SYNTAX_ERROR_REJECT = 2
    ABSTRACT_SYNTAX_ERROR_IGNORE_AND_NOTIFY = 3
    MESSAGE_NOT_COMPATIBLE_WITH_RECEIVER_STATE = 4
    SEMANTIC_ERROR = 5
    ABSTRACT_SYNTAX_ERROR_FALSELY_CONSTRUCTED_MESSAGE = 6


class ErrorCauseRicType(betterproto.Enum):
    UNSPECIFIED = 0
    RAN_FUNCTION_ID_INVALID = 1
    ACTION_NOT_SUPPORTED = 2
    EXCESSIVE_ACTIONS = 3
    DUPLICATE_ACTION = 4
    DUPLICATE_EVENT = 5
    FUNCTION_RESOURCE_LIMIT = 6
    REQUEST_ID_UNKNOWN = 7
    INCONSISTENT_ACTION_SUBSEQUENT_ACTION_SEQUENCE = 8
    CONTROL_MESSAGE_INVALID = 9
    CALL_PROCESS_ID_INVALID = 10


class ErrorCauseRicServiceType(betterproto.Enum):
    UNSPECIFIED = 0
    FUNCTION_NOT_REQUIRED = 1
    EXCESSIVE_FUNCTIONS = 2
    RIC_RESOURCE_LIMIT = 3


class ErrorCauseTransportType(betterproto.Enum):
    UNSPECIFIED = 0
    TRANSPORT_RESOURCE_UNAVAILABLE = 1


class ActionType(betterproto.Enum):
    ACTION_TYPE_REPORT = 0
    ACTION_TYPE_INSERT = 1
    ACTION_TYPE_POLICY = 2


class SubsequentActionType(betterproto.Enum):
    SUBSEQUENT_ACTION_TYPE_CONTINUE = 0
    SUBSEQUENT_ACTION_TYPE_WAIT = 1


class TimeToWait(betterproto.Enum):
    TIME_TO_WAIT_ZERO = 0
    TIME_TO_WAIT_W1MS = 1
    TIME_TO_WAIT_W2MS = 2
    TIME_TO_WAIT_W5MS = 3
    TIME_TO_WAIT_W10MS = 4
    TIME_TO_WAIT_W20MS = 5
    TIME_TO_WAIT_W30MS = 6
    TIME_TO_WAIT_W40MS = 7
    TIME_TO_WAIT_W50MS = 8
    TIME_TO_WAIT_W100MS = 9
    TIME_TO_WAIT_W200MS = 10
    TIME_TO_WAIT_W500MS = 11
    TIME_TO_WAIT_W1S = 12
    TIME_TO_WAIT_W2S = 13
    TIME_TO_WAIT_W5S = 14
    TIME_TO_WAIT_W10S = 15
    TIME_TO_WAIT_W20S = 16
    TIME_TO_WAIT_W60S = 17


@dataclass(eq=False, repr=False)
class RequestHeaders(betterproto.Message):
    app_id: str = betterproto.string_field(1)
    instance_id: str = betterproto.string_field(2)
    node_id: str = betterproto.string_field(3)
    service_model: "ServiceModel" = betterproto.message_field(4)
    encoding: "Encoding" = betterproto.enum_field(5)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ResponseHeaders(betterproto.Message):
    encoding: "Encoding" = betterproto.enum_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ServiceModel(betterproto.Message):
    name: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Error(betterproto.Message):
    """Error is an E2AP protocol error"""

    cause: "ErrorCause" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ErrorCause(betterproto.Message):
    unknown: "ErrorCauseUnknown" = betterproto.message_field(1, group="cause")
    protocol: "ErrorCauseProtocol" = betterproto.message_field(2, group="cause")
    ric: "ErrorCauseRic" = betterproto.message_field(3, group="cause")
    ric_service: "ErrorCauseRicService" = betterproto.message_field(4, group="cause")
    transport: "ErrorCauseTransport" = betterproto.message_field(5, group="cause")
    misc: "ErrorCauseMisc" = betterproto.message_field(6, group="cause")

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ErrorCauseUnknown(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ErrorCauseMisc(betterproto.Message):
    type: "ErrorCauseMiscType" = betterproto.enum_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ErrorCauseProtocol(betterproto.Message):
    type: "ErrorCauseProtocolType" = betterproto.enum_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ErrorCauseRic(betterproto.Message):
    type: "ErrorCauseRicType" = betterproto.enum_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ErrorCauseRicService(betterproto.Message):
    type: "ErrorCauseRicServiceType" = betterproto.enum_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ErrorCauseTransport(betterproto.Message):
    type: "ErrorCauseTransportType" = betterproto.enum_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ControlRequest(betterproto.Message):
    headers: "RequestHeaders" = betterproto.message_field(1)
    message: "ControlMessage" = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ControlMessage(betterproto.Message):
    header: bytes = betterproto.bytes_field(1)
    payload: bytes = betterproto.bytes_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ControlResponse(betterproto.Message):
    headers: "ResponseHeaders" = betterproto.message_field(1)
    outcome: "ControlOutcome" = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ControlOutcome(betterproto.Message):
    """Error is an E2AP protocol error"""

    payload: bytes = betterproto.bytes_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SubscribeRequest(betterproto.Message):
    headers: "RequestHeaders" = betterproto.message_field(1)
    subscription_id: str = betterproto.string_field(2)
    subscription_spec: "SubscriptionSpec" = betterproto.message_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SubscribeResponse(betterproto.Message):
    headers: "ResponseHeaders" = betterproto.message_field(1)
    ack: "Acknowledgement" = betterproto.message_field(2, group="message")
    indication: "Indication" = betterproto.message_field(3, group="message")

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UnsubscribeRequest(betterproto.Message):
    headers: "RequestHeaders" = betterproto.message_field(1)
    subscription_id: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UnsubscribeResponse(betterproto.Message):
    """Error is an E2AP protocol error"""

    headers: "ResponseHeaders" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SubscriptionSpec(betterproto.Message):
    event_trigger: "EventTrigger" = betterproto.message_field(1)
    actions: List["Action"] = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class EventTrigger(betterproto.Message):
    payload: bytes = betterproto.bytes_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Action(betterproto.Message):
    id: int = betterproto.int32_field(1)
    type: "ActionType" = betterproto.enum_field(2)
    payload: bytes = betterproto.bytes_field(3)
    subsequent_action: "SubsequentAction" = betterproto.message_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SubsequentAction(betterproto.Message):
    type: "SubsequentActionType" = betterproto.enum_field(1)
    time_to_wait: "TimeToWait" = betterproto.enum_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Acknowledgement(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Indication(betterproto.Message):
    header: bytes = betterproto.bytes_field(1)
    payload: bytes = betterproto.bytes_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListSubscriptionsRequest(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListSubscriptionsResponse(betterproto.Message):
    subscription: "Subscription" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchSubscriptionsRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchSubscriptionsResponse(betterproto.Message):
    subscription: "Subscription" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SubscriptionMeta(betterproto.Message):
    app_id: str = betterproto.string_field(1)
    instance_id: str = betterproto.string_field(2)
    node_id: str = betterproto.string_field(3)
    subscription_id: str = betterproto.string_field(4)
    service_model: "ServiceModel" = betterproto.message_field(5)
    encoding: "Encoding" = betterproto.enum_field(6)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Subscription(betterproto.Message):
    meta: "SubscriptionMeta" = betterproto.message_field(1)
    spec: "SubscriptionSpec" = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


class ControlServiceStub(betterproto.ServiceStub):
    async def control(
        self, *, headers: "RequestHeaders" = None, message: "ControlMessage" = None
    ) -> "ControlResponse":

        request = ControlRequest()
        if headers is not None:
            request.headers = headers
        if message is not None:
            request.message = message

        return await self._unary_unary(
            "/onos.e2t.e2.v1beta2.ControlService/Control", request, ControlResponse
        )


class SubscriptionServiceStub(betterproto.ServiceStub):
    async def subscribe(
        self,
        *,
        headers: "RequestHeaders" = None,
        subscription_id: str = "",
        subscription_spec: "SubscriptionSpec" = None,
    ) -> AsyncIterator["SubscribeResponse"]:

        request = SubscribeRequest()
        if headers is not None:
            request.headers = headers
        request.subscription_id = subscription_id
        if subscription_spec is not None:
            request.subscription_spec = subscription_spec

        async for response in self._unary_stream(
            "/onos.e2t.e2.v1beta2.SubscriptionService/Subscribe",
            request,
            SubscribeResponse,
        ):
            yield response

    async def unsubscribe(
        self, *, headers: "RequestHeaders" = None, subscription_id: str = ""
    ) -> "UnsubscribeResponse":

        request = UnsubscribeRequest()
        if headers is not None:
            request.headers = headers
        request.subscription_id = subscription_id

        return await self._unary_unary(
            "/onos.e2t.e2.v1beta2.SubscriptionService/Unsubscribe",
            request,
            UnsubscribeResponse,
        )


class SubscriptionAdminServiceStub(betterproto.ServiceStub):
    async def list_subscriptions(self) -> AsyncIterator["ListSubscriptionsResponse"]:

        request = ListSubscriptionsRequest()

        async for response in self._unary_stream(
            "/onos.e2t.e2.v1beta2.SubscriptionAdminService/ListSubscriptions",
            request,
            ListSubscriptionsResponse,
        ):
            yield response

    async def watch_subscriptions(
        self, *, no_replay: bool = False
    ) -> AsyncIterator["WatchSubscriptionsResponse"]:

        request = WatchSubscriptionsRequest()
        request.no_replay = no_replay

        async for response in self._unary_stream(
            "/onos.e2t.e2.v1beta2.SubscriptionAdminService/WatchSubscriptions",
            request,
            WatchSubscriptionsResponse,
        ):
            yield response
